import collections
from abjad.tools import durationtools


def resolve_overlaps_between_nonoverlapping_trees(trees, minimum_duration=None):
    '''Create a nonoverlapping TimeIntervalTree from `trees`.
    Intervals in higher-indexed trees in `trees` only appear in part or whole where they do not
    overlap intervals from starter-indexed trees ::

        >>> from abjad.tools.timeintervaltools import TimeInterval
        >>> from abjad.tools.timeintervaltools import TimeIntervalTree

    ::

        >>> a = TimeIntervalTree(TimeInterval(0, 4, {'a': 1}))
        >>> b = TimeIntervalTree(TimeInterval(1, 5, {'b': 2}))
        >>> c = TimeIntervalTree(TimeInterval(2, 6, {'c': 3}))
        >>> d = TimeIntervalTree(TimeInterval(1, 3, {'d': 4}))
        >>> timeintervaltools.resolve_overlaps_between_nonoverlapping_trees([a, b, c, d])
        TimeIntervalTree([
            TimeInterval(Offset(0, 1), Offset(4, 1), {'a': 1}),
            TimeInterval(Offset(4, 1), Offset(5, 1), {'b': 2}),
            TimeInterval(Offset(5, 1), Offset(6, 1), {'c': 3})
        ])

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    assert isinstance(trees, collections.Iterable) and len(trees) \
        and all([isinstance(x, timeintervaltools.TimeIntervalTree) for x in trees]) \
        and all([timeintervaltools.all_intervals_are_nonoverlapping(x) for x in trees])

    if minimum_duration is None:
        minimum_duration = durationtools.Duration(0)
    else:
        minimum_duration = durationtools.Duration(minimum_duration)
        assert 0 < minimum_duration

    rtree = trees[0]
    for tree in trees[1:]:
        xor = timeintervaltools.compute_logical_xor_of_intervals([tree, rtree])
        masked = timeintervaltools.mask_intervals_with_intervals(tree, xor)
        rtree._insert([x for x in masked if minimum_duration <= x.duration])

    assert timeintervaltools.all_intervals_are_nonoverlapping(rtree)

    return rtree



import copy
from abjad.tools.leaftools.Leaf import Leaf


class Skip(Leaf):
    '''Abjad model of a LilyPond skip:

    ::

        >>> skiptools.Skip((3, 16))
        Skip('s8.')

    Return Skip instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    _default_positional_input_arguments = (repr('s4'), )

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        from abjad.tools import lilypondparsertools

        if len(args) == 1 and isinstance(args[0], str):
            input = '{{ {} }}'.format(args[0])
            parsed = lilypondparsertools.LilyPondParser()(input)
            assert len(parsed) == 1 and isinstance(parsed[0], Leaf)
            args = [parsed[0]]
            #written_duration = args[0].strip('s')
            #lilypond_multiplier = None

        if len(args) == 1 and isinstance(args[0], Leaf):
            leaf = args[0]
            written_duration = leaf.written_duration
            lilypond_multiplier = leaf.duration_multiplier
            self._copy_override_and_set_from_leaf(leaf)
        elif len(args) == 1 and not isinstance(args[0], str):
            written_duration = args[0]
            lilypond_multiplier = None
        elif len(args) == 2:
            written_duration, lilypond_multiplier = args
        else:
            raise ValueError('can not initialize rest from "%s".' % str(args))

        Leaf.__init__(self, written_duration, lilypond_multiplier)
        self._initialize_keyword_values(**kwargs)

    # SPECIAL METHODS #

    #__deepcopy__ = __copy__

    ### PRIVATE PROPERTIES ###

    @property
    def _body(self):
        result = []
        result.append('s%s' % self._formatted_duration)
        return result

    @property
    def _compact_representation(self):
        return 's%s' % self._formatted_duration



import py.test
from abjad import *


def test_TreeContainer_name_01():

    foo = datastructuretools.TreeContainer(name='foo')
    bar = datastructuretools.TreeContainer(name='bar')
    baz = datastructuretools.TreeContainer(name='baz')
    quux = datastructuretools.TreeContainer(name='quux')
    quux2 = datastructuretools.TreeContainer(name='quux')

    foo.append(bar)
    bar.extend([baz, quux])
    baz.append(quux2)

    '''
    TreeContainer(
        children=(
            TreeContainer(
                children=(
                    TreeContainer(
                        children=(
                            TreeContainer(
                                name='quux'
                                ),
                            ),
                        name='baz'
                        ),
                    TreeContainer(
                        name='quux'
                        )
                    ),
                name='bar'
                ),
            ),
        name='foo'
        )
    '''

    assert foo['bar'] is bar
    assert foo['baz'] is baz
    assert py.test.raises(ValueError, "foo['quux']")

    quux2.name = 'wux'

    assert foo['quux'] is quux
    assert foo['wux'] is quux2




import os


# TODO: make public and possibly improve function name
def _run_lilypond(lilypond_file_name, lilypond_path):
    from abjad.tools import iotools

    if not lilypond_path:
        lilypond_path = 'lilypond'
    command = '{} -dno-point-and-click {} > lily.log 2>&1'.format(lilypond_path, lilypond_file_name)
    iotools.spawn_subprocess(command)
    postscript_file_name = lilypond_file_name.replace('.ly', '.ps')
    try:
        os.remove(postscript_file_name)
    except OSError:
        # no such file...
        pass



from abjad.tools import leaftools


def make_tied_rest(duration, decrease_durations_monotonically=True, tie_parts=False):
    '''Returns a list of rests to fill given duration.
    
    Tie rest parts when ``tie_parts=True``.
    '''
    from abjad.tools import resttools

    return leaftools.make_tied_leaf(
        resttools.Rest, duration, 
        decrease_durations_monotonically=decrease_durations_monotonically, 
        pitches=None, tie_parts=tie_parts)



from abjad.tools.contexttools.Context import Context


class Voice(Context):
    r'''Abjad model of a voice:

    ::

        >>> voice = Voice("c'8 d'8 e'8 f'8")

    ::

        >>> voice
        Voice{4}

    ::

        >>> f(voice)
        \new Voice {
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> show(voice) # doctest: +SKIP  

    Return voice instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, music=None, context_name='Voice', name=None):
        Context.__init__(self, music=music)
        self._initialize_keyword_values(context_name=context_name, name=name)



import abc
from abjad.tools import componenttools
from abjad.tools import formattools
from abjad.tools.containertools.Container import Container
import copy


class Context(Container):
    '''.. versionadded:: 1.0

    Abjad model of a horizontal layer of music.

    ::

        >>> context = contexttools.Context(
        ... name='MeterVoice', context_name='TimeSignatureContext')

    ::

        >>> context
        TimeSignatureContext-"MeterVoice"{}

    ::

        >>> f(context)
        \context TimeSignatureContext = "MeterVoice" {
        }

    Return context object.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta 
    __slots__ = ('_context_name', '_engraver_consists', '_engraver_removals',
        '_is_nonsemantic', '_name', )

    ### INITIALIZER ###

    def __init__(self, music=None, context_name='Context', name=None):
        Container.__init__(self, music=music)
        self.context_name = context_name
        self._engraver_consists = []
        self._engraver_removals = []
        self._name = None
        self.name = name
        self.is_nonsemantic = False

    ### SPECIAL METHODS ###

    def __repr__(self):
        '''.. versionchanged:: 2.0

        Named contexts now print name at the interpreter.
        '''
        if 0 < len(self):
            summary = str(len(self))
        else:
            summary = ''
        if self.is_parallel:
            open_bracket_string, close_bracket_string = '<<', '>>'
        else:
            open_bracket_string, close_bracket_string = '{', '}'
        name = self.name
        if name is not None:
            name = '-"%s"' % name
        else:
            name = ''
        return '%s%s%s%s%s' % (self.context_name, name, open_bracket_string, summary, close_bracket_string)

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        return self._format_component(pieces=True)

    ### PRIVATE METHODS ###

    def _copy_with_marks_but_without_children_or_spanners(self):
        new = Container._copy_with_marks_but_without_children_or_spanners(self)
        new._engraver_consists = copy.copy(self.engraver_consists)
        new._engraver_removals = copy.copy(self.engraver_removals)
        new.name = copy.copy(self.name)
        new.is_nonsemantic = copy.copy(self.is_nonsemantic)
        return new
        return new

    def _format_closing_slot(context, format_contributions):
        result = []
        result.append(['context marks', format_contributions.get('closing', {}).get('context marks', [])])
        result.append(['lilypond command marks', 
            format_contributions.get('closing', {}).get('lilypond command marks', [])])
        result.append(['comments', format_contributions.get('closing', {}).get('comments', [])])
        return context._format_slot_contributions_with_indent(result)

    def _format_engraver_consists(self):
        result = []
        for engraver in self.engraver_consists:
            result.append(r'\consists %s' % engraver)
        return result

    def _format_engraver_removals(self):
        result = []
        for engraver in self.engraver_removals:
            result.append(r'\remove %s' % engraver)
        return result

    def _format_invocation(self):
        if self.name is not None:
            return r'\context %s = "%s"' % (self.context_name, self.name)
        else:
            return r'\new %s' % self.context_name

    def _format_open_brackets_slot(context, format_contributions):
        result = []
        if context.is_parallel:
            brackets_open = ['<<']
        else:
            brackets_open = ['{']
        engraver_removals = context._format_engraver_removals()
        engraver_consists = context._format_engraver_consists()
        overrides = format_contributions.get('grob overrides', [])
        settings = format_contributions.get('context settings', [])
        if engraver_removals or engraver_consists or overrides or settings:
            contributions = [context._format_invocation() + r' \with {']
            result.append([('context_brackets', 'open'), contributions])
            contributions = ['\t' + x for x in engraver_removals]
            result.append([('engraver removals', 'engraver_removals'), contributions])
            contributions = ['\t' + x for x in engraver_consists]
            result.append([('engraver consists', 'engraver_consists'), contributions])
            contributions = ['\t' + x for x in overrides]
            result.append([('overrides', 'overrides'), contributions])
            contributions = ['\t' + x for x in settings]
            result.append([('settings', 'settings'), contributions])
            contributions = ['} %s' % brackets_open[0]]
            result.append([('context_brackets', 'open'), contributions])
        else:
            contributions = [context._format_invocation() + ' %s' % brackets_open[0]]
            result.append([('context_brackets', 'open'), contributions])
        return tuple(result)

    def _format_opening_slot(context, format_contributions):
        result = []
        result.append(['comments', format_contributions.get('opening', {}).get('comments', [])])
        result.append(['context marks', format_contributions.get('opening', {}).get('context marks', [])])
        result.append(['lilypond command marks', 
            format_contributions.get('opening', {}).get('lilypond command marks', [])])
        return context._format_slot_contributions_with_indent(result)

    def _initialize_keyword_values(self, **kwargs):
        if 'context_name' in kwargs:
            self.context_name = kwargs['context_name']
            del(kwargs['context_name'])
        if 'name' in kwargs:
            self.name = kwargs['name']
            del(kwargs['name'])
        Container._initialize_keyword_values(self, **kwargs)

    ### PUBLIC PROPERTIES ###

    @apply
    def context_name():
        def fget(self):
            '''Read / write name of context as a string.'''
            return self._context_name
        def fset(self, arg):
            assert isinstance(arg, str)
            self._context_name = arg
        return property(**locals())

    @property
    def engraver_consists(self):
        r'''.. versionadded:: 2.0

        Unordered set of LilyPond engravers to include in context definition.

        Manage with add, update, other standard set commands:

        ::

            >>> staff = Staff([])
            >>> staff.engraver_consists.append('Horizontal_bracket_engraver')
            >>> f(staff)
            \new Staff \with {
                \consists Horizontal_bracket_engraver
            } {
            }

        '''
        return self._engraver_consists

    @property
    def engraver_removals(self):
        r'''.. versionadded:: 2.0

        Unordered set of LilyPond engravers to remove from context.

        Manage with add, update, other standard set commands. ::

            >>> staff = Staff([])
            >>> staff.engraver_removals.append('Time_signature_engraver')
            >>> f(staff)
            \new Staff \with {
                \remove Time_signature_engraver
            } {
            }

        '''
        return self._engraver_removals

    @apply
    def is_nonsemantic():
        def fget(self):
            r'''Set indicator of nonsemantic voice:

            ::

                >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(
                ...     [(1, 8), (5, 16), (5, 16)])
                >>> voice = Voice(measures)
                >>> voice.name = 'HiddenTimeSignatureVoice'

            ::

                >>> voice.is_nonsemantic = True

            ::

                >>> f(voice)
                \context Voice = "HiddenTimeSignatureVoice" {
                    {
                        \time 1/8
                        s1 * 1/8
                    }
                    {
                        \time 5/16
                        s1 * 5/16
                    }
                    {
                        s1 * 5/16
                    }
                }

            ::

                >>> voice.is_nonsemantic
                True

            Get indicator of nonsemantic voice:

            ::

                >>> voice = Voice([])

            ::

                >>> voice.is_nonsemantic
                False

            Return boolean.

            The intent of this read / write attribute is to allow composers to tag invisible
            voices used to house time signatures indications, bar number directives or other
            pieces of score-global non-musical information. Such nonsemantic voices can then
            be omitted from voice interation and other functions.
            '''
            return self._is_nonsemantic
        def fset(self, arg):
            if not isinstance(arg, bool):
                raise TypeError
            self._is_nonsemantic = arg
        return property(**locals())

    @property
    def is_semantic(self):
        return not self.is_nonsemantic

    @property
    def lilypond_format(self):
        self._update_marks_of_entire_score_tree_if_necessary()
        return self._format_component()

    @apply
    def name():
        def fget(self):
            '''Read-write name of context. Must be string or none.
            '''
            return self._name
        def fset(self, arg):
            assert isinstance(arg, (str, type(None)))
            parentage = componenttools.get_proper_parentage_of_component(self)
            old_name = self._name
            for parent in parentage:
                named_children = parent._named_children
                if old_name is not None:
                    named_children[old_name].remove(self)
                    if not named_children[old_name]:
                        del named_children[old_name]
                if arg is not None:
                    if arg not in named_children:
                        named_children[arg] = [self]
                    else:
                        named_children[arg].append(self)
            self._name = arg
        return property(**locals())



import abc
import copy
from abjad.tools import durationtools
from abjad.tools import sequencetools
from abjad.tools import tuplettools
from abjad.tools.abctools.AbjadObject import AbjadObject


class RhythmMaker(AbjadObject):
    '''.. versionadded:: 2.8

    Rhythm maker abstract base class.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, beam_each_cell=True, beam_cells_together=False):
        self.beam_each_cell = beam_each_cell
        self.beam_cells_together = beam_cells_together

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        duration_pairs = [durationtools.Duration(x).pair for x in divisions]
        seeds = self._none_to_new_list(seeds)
        return duration_pairs, seeds

    def __repr__(self):
        if getattr(self, 'name', None) is not None:
            return '{}({!r})'.format(self._class_name, self.name)
        return AbjadObject.__repr__(self)

    ### PRIVATE METHODS ###

    def _make_secondary_duration_pairs(self, duration_pairs, secondary_divisions):
        if not secondary_divisions:
            return duration_pairs[:]
        numerators = [duration_pair.numerator for duration_pair in duration_pairs]
        secondary_numerators = sequencetools.split_sequence_by_weights(
            numerators, secondary_divisions, cyclic=True, overhang=True)
        secondary_numerators = sequencetools.flatten_sequence(secondary_numerators)
        denominator = duration_pairs[0].denominator
        secondary_duration_pairs = [(n, denominator) for n in secondary_numerators]
        return secondary_duration_pairs

    def _make_tuplets(self, duration_pairs, leaf_lists):
        assert len(duration_pairs) == len(leaf_lists)
        tuplets = []
        for duration_pair, leaf_list in zip(duration_pairs, leaf_lists):
            tuplet = tuplettools.FixedDurationTuplet(duration_pair, leaf_list)
            tuplets.append(tuplet)
        return tuplets

    def _none_to_new_list(self, expr):
        if expr is None:
            return []
        return expr

    def _none_to_trivial_helper(self, expr):
        if expr is None:
            return self._trivial_helper
        return expr

    def _scale_talee(self, duration_pairs, talea_denominator, talee):
        dummy_duration_pair = (1, talea_denominator)
        duration_pairs.append(dummy_duration_pair)
        duration_pairs = durationtools.durations_to_nonreduced_fractions_with_common_denominator(
            duration_pairs)
        dummy_duration_pair = duration_pairs.pop()
        lcd = dummy_duration_pair.denominator
        multiplier = lcd / talea_denominator
        scaled_talee = []
        for talea in talee:
            talea = sequencetools.CyclicTuple([multiplier * x for x in talea])
            scaled_talee.append(talea)
        result = [duration_pairs, lcd]
        result.extend(scaled_talee)
        return tuple(result)

    def _sequence_to_ellipsized_string(self, sequence):
        if not sequence:
            return '[]'
        if len(sequence) <= 4:
            result = ', '.join([str(x) for x in sequence])
        else:
            result = ', '.join([str(x) for x in sequence[:4]])
            result += ', ...'
        result = '[$%s$]' % result
        return result

    def _trivial_helper(self, talea, seeds):
        return talea

    ### PUBLIC METHODS ###

    def new(self, **kwargs):
        r'''.. versionadded:: 2.11

        Copy rhythm-maker.

        Set keyword arguments on copied rhythm-maker::

            >>> maker = rhythmmakertools.NoteRhythmMaker()

        ::

            >>> divisions = [(5, 16), (3, 8)]
            >>> leaf_lists = maker.new(decrease_durations_monotonically=False)(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)

        ::

            >>> staff = Staff(measuretools.make_measures_with_full_measure_spacer_skips(divisions))
            >>> measures = measuretools.replace_contents_of_measures_in_expr(staff, leaves)

        ::

            >>> f(staff)
            \new Staff {
                {
                    \time 5/16
                    c'16 ~
                    c'4
                }
                {
                    \time 3/8
                    c'4.
                }
            }

        Return copied rhythm-maker.
        '''
        new = copy.deepcopy(self)
        for key, value in kwargs.iteritems():
            setattr(new, key, value)
        return new
 
    def reverse(self):
        '''.. versionadded:: 2.10

        Reverse rhythm-maker.

        .. note:: method is provisional.

        Defined equal to exact copy of rhythm-maker.

        This is the fallback for child classes.

        Directed rhythm-maker child classes should override this method.

        Return newly constructed rhythm-maker.
        '''
        new = copy.deepcopy(self)
        return new



from abjad.tools import componenttools


def all_are_rests(expr):
    '''.. versionadded:: 2.6

    True when `expr` is a sequence of Abjad rests::

        >>> rests = [Rest('r4'), Rest('r4'), Rest('r4')]

    ::

        >>> resttools.all_are_rests(rests)
        True

    True when `expr` is an empty sequence::

        >>> resttools.all_are_rests([])
        True

    Otherwise false::

        >>> resttools.all_are_rests('foo')
        False

    Return boolean.

    Function wraps ``componenttools.all_are_components()``.
    '''
    from abjad.tools import resttools

    return componenttools.all_are_components(expr, klasses=(resttools.Rest,))



from abjad import *


def test_MetricalHierarchyInventory___contains___01():

    rtm = '(6/8 ((3/8 (1/8 1/8 1/8)) (3/8 (1/8 1/8 1/8))))'

    inventory = timesignaturetools.MetricalHierarchyInventory([(4, 4), (3, 4), rtm])

    assert (4, 4) in inventory
    assert timesignaturetools.MetricalHierarchy((4, 4)) in inventory

    assert (3, 4) in inventory
    assert timesignaturetools.MetricalHierarchy((3, 4)) in inventory

    assert rtm in inventory
    assert timesignaturetools.MetricalHierarchy(rtm) in inventory



from abjad.tools import stringtools


# TODO: make public and move somewhere
def _make_lilypond_revert_string(grob_name, grob_attribute, context_name=None):
    '''.. versionadded:: 2.0

    Make LilyPond revert string.
    '''
    from _format_lilypond_attribute import _format_lilypond_attribute
    from _format_lilypond_value import _format_lilypond_value

    # parse input strings
    grob_name = stringtools.underscore_delimited_lowercase_to_uppercamelcase(grob_name)
    grob_attribute = _format_lilypond_attribute(grob_attribute)
    if context_name is not None:
        context_prefix = stringtools.underscore_delimited_lowercase_to_uppercamelcase(context_name)
        context_prefix += '.'
    else:
        context_prefix = ''

    # return override string
    return r'\revert %s%s %s' % (context_prefix, grob_name, grob_attribute)



import docutils
import hashlib
import multiprocessing
import os
import pickle
import posixpath
import shutil
import sphinx
import subprocess
import tempfile
from abjad import ABJCFG
from abjad.tools import documentationtools
from abjad.tools import sequencetools


class abjad_book_block(docutils.nodes.General, docutils.nodes.Element):
    pass


class abjad_literal_block(docutils.nodes.General, docutils.nodes.Element):
    pass


class AbjadBookDirective(sphinx.util.compat.Directive):
    has_content = True
    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        'errors-ok': docutils.parsers.rst.directives.flag,
        'hidden': docutils.parsers.rst.directives.flag,
        'strip-prompt': docutils.parsers.rst.directives.flag,
    }
    def run(self):
        self.assert_has_content()
        code = u'\n'.join(self.content)
        literal = abjad_literal_block(code, code)
        literal['errors-ok'] = 'errors-ok' in self.options
        literal['hidden'] = 'hidden' in self.options
        literal['strip-prompt'] = 'strip-prompt' in self.options
        #literal = docutils.nodes.literal_block(code, code)
        sphinx.util.nodes.set_source_info(self, literal)
        return [literal]


class ShellDirective(sphinx.util.compat.Directive):
    has_content = True
    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = False
    option_spec = {}
    def run(self):
        self.assert_has_content()
        original_directory = os.path.abspath(os.path.curdir)
        os.chdir(ABJCFG.ABJAD_PATH)
        result = []
        for line in self.content:
            prompt = '{}$ '.format(os.path.basename(os.path.abspath(os.path.curdir)))
            prompt += line
            result.append(prompt)
            proc = subprocess.Popen(line.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            out, err = proc.communicate()
            result.extend(out.splitlines())
            result.extend(err.splitlines())
        code = u'\n'.join(result)
        literal = docutils.nodes.literal_block(code, code)
        literal['language'] = 'bash'
        sphinx.util.nodes.set_source_info(self, literal)
        os.chdir(original_directory)
        return [literal]


def on_builder_inited(app):
    tmp_directory = app.builder._abjad_book_tempdir = \
        os.path.abspath(tempfile.mkdtemp(dir=app.builder.outdir))
    if hasattr(app.builder, 'imgpath'):
        img_directory = os.path.join(app.builder.outdir, '_images')
        if not os.path.exists(img_directory):
            os.makedirs(img_directory)


def rewrite_literal_block_line(line):
    if line.strip().startswith(('f(', 'play(', 'print ', 'redo(', 'z(', 'iotools.log(')):
        return '', False
    elif not line.startswith(('show(', 'iotools.graph')):
        return line, False
    if line.startswith('show('):
        object_name = line[5:]
        kind = 'lilypond'
    elif line.startswith('iotools.graph('):
        object_name = line[14:]
        kind = 'graphviz'
    object_name = object_name.rpartition(')')[0]
    if ')' in object_name:
        object_name = object_name.rpartition(')')[0] + ')'
    elif ',' in object_name:
        object_name = object_name.rpartition(',')[0]
    object_name = object_name.strip()
    if kind == 'lilypond':
        return '__abjad_book__ = ({!r}, ' \
            'documentationtools.make_reference_manual_lilypond_file({}).lilypond_format)'.format(
            kind, object_name), True
    elif kind == 'graphviz':
        return '__abjad_book__ = ({!r}, ' \
            'documentationtools.make_reference_manual_graphviz_graph({}).graphviz_format)'.format(
            kind, object_name), True


def scan_doctree_for_literal_blocks(doctree):
    def is_valid_node(node):
        if isinstance(node, 
            (docutils.nodes.literal_block, docutils.nodes.doctest_block)
            ):
            return True
        return False
    should_process = False
    literal_blocks = [x for x in doctree.traverse(is_valid_node)]
    literal_lines = []
    for literal_block in literal_blocks:
        lines = []
        for i, line in enumerate(literal_block[0].splitlines()):
            if line.startswith(('>>>', '... ')):
                rewritten_line, has_image = rewrite_literal_block_line(line[4:])
                should_process = should_process or has_image
                lines.append((i, rewritten_line))
        literal_lines.append(tuple(lines))
    return zip(literal_blocks, literal_lines), should_process
     

def scan_doctree_for_abjad_literal_blocks(doctree):
    abjad_literal_blocks = [x for x in doctree.traverse(abjad_literal_block)]
    should_process = 0 < len(abjad_literal_blocks)
    return abjad_literal_blocks, should_process


def process_literal_block_pairs(literal_block_pairs):
    environment = {'__builtins__': __builtins__}
    exec('from abjad import *\n', environment)
    for literal_block, all_lines in literal_block_pairs:
        original_lines = literal_block[0].splitlines()
        replacement_blocks = []
        lines_to_execute = []
        previous_line_number = 0
        for i, line in all_lines:
            lines_to_execute.append(line)
            if line.startswith('__abjad_book__ ='):
                if '__abjad_book__' in environment:
                    del(environment['__abjad_book__'])
                try:
                    exec('\n'.join(lines_to_execute), environment)
                except:
                    pass
                kind, code = environment['__abjad_book__']
                new_abjad_book_block = abjad_book_block()
                new_abjad_book_block['kind'] = kind
                new_abjad_book_block['code'] = code
                text = '\n'.join(original_lines[previous_line_number:i + 1])
                new_literal_block = literal_block.deepcopy()
                new_literal_block.rawsource = text
                new_literal_block[0].rawsource = text
                new_literal_block[0].text = text
                replacement_blocks.extend([new_literal_block, new_abjad_book_block])
                lines_to_execute = []
                previous_line_number = i + 1
        if lines_to_execute:
            try:
                exec('\n'.join(lines_to_execute), environment)
            except:
                pass
            if replacement_blocks:
                text = '\n'.join(original_lines[previous_line_number:])
                new_literal_block = literal_block.deepcopy()
                new_literal_block.rawsource = text
                new_literal_block[0].rawsource = text
                new_literal_block[0].text = text
                replacement_blocks.append(new_literal_block)
        if replacement_blocks:
            literal_block.replace_self(replacement_blocks)


def process_abjad_literal_blocks(abjad_literal_blocks):
    # setup pipe
    pipe = documentationtools.Pipe()
    pipe.read_wait()
    pipe.write('from abjad import *\n')
    pipe.read_wait()

    for abjad_literal_block in abjad_literal_blocks:
        pass
            
    # cleanup pipe
    pipe.write('quit()\n')
    pipe.close()


def on_doctree_read(app, doctree):
    transform_path = app.config.abjad_book_transform_path
    docname = doctree['source'][:-4].partition(app.srcdir)[-1][1:]
    if not docname.startswith(transform_path):
        return
    result_a = scan_doctree_for_literal_blocks(doctree)
    result_b = scan_doctree_for_abjad_literal_blocks(doctree)
    should_process = result_a[1] or result_b[1]
    if not should_process:
        return
    literal_block_pairs = result_a[0]
    abjad_literal_blocks = result_b[0]
    process_literal_block_pairs(literal_block_pairs)
    process_abjad_literal_blocks(abjad_literal_blocks)


def on_build_finished(app, exc):
    if os.path.exists(app.builder._abjad_book_tempdir):
        shutil.rmtree(app.builder._abjad_book_tempdir)


def render_graphviz_image(self, code, paths, file_format='png',
    is_latex=False, is_pickled=False, keep_original=False):
    assert file_format in ('png', 'pdf')
    primary_path = paths['primary_absolute_path']
    secondary_path = paths['secondary_absolute_path']
    tmp_path = os.path.join(self.builder._abjad_book_tempdir,
        os.path.basename(os.path.splitext(primary_path)[0])) + '.dot'
    # if we pickled a documentationtools.GraphvizGraph instance
    # in order to support conditional reformatting for LaTeX vs HTML...
    if is_pickled:
        graph = pickle.loads(code)
        if is_latex:
            graph.attributes['fontsize'] = 10
            graph.attributes['ranksep'] = 0.25
            graph.attributes['ratio'] = 'compress'
            graph.attributes['size'] = 6.5
            graph.node_attributes['fontsize'] = 8
            graph.node_attributes['margin'] = (0.06, 0.06)
        code = graph.unflattened_graphviz_format
    with open(tmp_path, 'w') as f:
        f.write(code)
    commands = []
    if file_format == 'pdf':
        commands.append('dot -Tpdf -o {} {}'.format(primary_path, tmp_path))
        commands.append('pdfcrop {} {}'.format(primary_path, primary_path))
    elif file_format == 'png':
        commands.append('dot -Tpdf -o {} {}'.format(secondary_path, tmp_path))
        commands.append('dot -Tpng -o {} {}'.format(primary_path, tmp_path))
        commands.append('convert -debug Exception -trim -resize 75% -resize 460x9999">" {} {}'.format(
            primary_path, primary_path))
    for command in commands:
        subprocess.call(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        #subprocess.call(command, shell=True)
    if not keep_original and os.path.exists(secondary_path):
        os.remove(secondary_path)


def render_lilypond_image(self, code, paths, file_format='png', keep_original=False):
    assert file_format in ('png', 'pdf')
    absolute_path = paths['primary_absolute_path']
    # LilyPond insists on appending an extension, even if you already did it yourself.
    abs_path = os.path.splitext(absolute_path)[0]
    tmp_path = os.path.join(self.builder._abjad_book_tempdir,
        os.path.basename(os.path.splitext(absolute_path)[0])) + '.ly'
    with open(tmp_path, 'w') as f:
        f.write(code)
    commands = []
    if file_format == 'png':
        commands.append('lilypond --png -dresolution=300 -o {} {}'.format(abs_path, tmp_path))
        commands.append('convert -trim -resample 50%% {} {}'.format(absolute_path, absolute_path))
    elif file_format == 'pdf':
        commands.append('lilypond -o {} {}'.format(abs_path, tmp_path))
        commands.append('pdfcrop {} {}'.format(absolute_path, absolute_path))
    for command in commands:
        subprocess.call(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)


def render_abjad_book_node(self, node, file_format='png', linked=False):
    code = node['code']
    kind = node['kind']
    keep_original = node.get('keep_original', False)
    is_pickled = node.get('is_pickled', False)
    suffix = 'original'
    hashkey = code.encode('utf-8') + kind
    hexdigest = hashlib.sha1(hashkey).hexdigest()
    # primary in the target format, but secondary always in full-quality PDF
    primary_file_name = '{}-{}.{}'.format(
        kind, hexdigest, file_format)
    secondary_file_name = '{}-{}-{}.pdf'.format(
        kind, hexdigest, suffix)
    paths = {}
    if hasattr(self.builder, 'imgpath'): # HTML
        is_latex = False
        img_directory = os.path.join(self.builder.outdir, '_images')
        if not os.path.exists(img_directory):
            os.makedirs(img_directory)
        paths['primary_relative_path'] = posixpath.join(
            self.builder.imgpath, primary_file_name)
        paths['primary_absolute_path'] = os.path.join(
            self.builder.outdir, '_images', primary_file_name)
        paths['secondary_relative_path'] = posixpath.join(
            self.builder.imgpath, secondary_file_name)
        paths['secondary_absolute_path'] = os.path.join(
            self.builder.outdir, '_images', secondary_file_name)
    else: # LaTeX
        is_latex = True
        paths['primary_relative_path'] = primary_file_name
        paths['primary_absolute_path'] = os.path.join(
            self.builder.outdir, primary_file_name)
        paths['secondary_relative_path'] = secondary_file_name
        paths['secondary_absolute_path'] = os.path.join(
            self.builder.outdir, secondary_file_name)
    if os.path.isfile(paths['primary_absolute_path']):
        return paths
    if kind == 'lilypond':
        render_lilypond_image(self, code, paths, file_format,
            keep_original=keep_original)
    elif kind == 'graphviz':
        render_graphviz_image(self, code, paths, file_format,
            is_latex=is_latex,
            is_pickled=is_pickled,
            keep_original=keep_original,
            )
    return paths


def visit_abjad_book_html(self, node):
    paths = render_abjad_book_node(
        self, node, file_format='png',
        )
    wrapper = 'p'
    alt = self.encode(node['code']).strip()
    self.body.append(self.starttag(node, wrapper, CLASS='abjad_book'))
    img = '<img src="{}" alt="{}" />'.format(
        paths['primary_relative_path'], alt)
    if node.get('keep_original', False):
        figure = '<figure>{anchor}<figcaption>{caption}</figcaption></figure>' 
        anchor = '<a href="{}" alt="click for high-quality original">{}</a>'.format(
            paths["secondary_relative_path"], img)
        caption = '<a href="{}">(click for high-quality image)</a>'.format(
            paths["secondary_relative_path"])
        self.body.append(figure.format(anchor=anchor, caption=caption))
    else:
        self.body.append(img)
    self.body.append('</{}>\n'.format(wrapper))
    raise docutils.nodes.SkipNode


def visit_abjad_book_latex(self, node):
    paths = render_abjad_book_node(
        self, node, file_format='pdf',
        )
    self.body.append('\n\\includegraphics{' + paths['primary_relative_path'] + '}\n')
    raise docutils.nodes.SkipNode


def setup(app):
    app.add_directive('abjad', AbjadBookDirective)
    app.add_directive('shell', ShellDirective)
    app.add_node(abjad_book_block,
        html=(visit_abjad_book_html, None),
        latex=(visit_abjad_book_latex, None),
        # we can add more rendering options later
        #texinfo=(visit_abjad_book_texinfo, None),
        #text=(visit_abjad_book_text, None),
        #man=(visit_abjad_book_man, None)
        )
    app.connect('builder-inited', on_builder_inited)
    app.connect('doctree-read', on_doctree_read)
    app.connect('build-finished', on_build_finished)
    app.add_config_value('abjad_book_transform_path', 'api/tools/', 'env')




from abjad import *


def test_iterationtools_iterate_measures_in_expr_01():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    r'''
    \new Staff {
        {
            \time 2/8
            c'8
            d'8
        }
        {
            \time 2/8
            e'8
            f'8
        }
        {
            \time 2/8
            g'8
            a'8
        }
    }
    '''

    measures = list(iterationtools.iterate_measures_in_expr(staff, reverse=True))

    assert measures[0] is staff[2]
    assert measures[1] is staff[1]
    assert measures[2] is staff[0]


def test_iterationtools_iterate_measures_in_expr_02():
    '''Optional start and stop keyword paramters.'''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    measures = list(iterationtools.iterate_measures_in_expr(staff, start=1, reverse=True))
    assert measures[0] is staff[1]
    assert measures[1] is staff[0]
    assert len(measures) == 2

    measures = list(iterationtools.iterate_measures_in_expr(staff, stop=2, reverse=True))
    assert measures[0] is staff[2]
    assert measures[1] is staff[1]
    assert len(measures) == 2


def test_iterationtools_iterate_measures_in_expr_03():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    r'''
    \new Staff {
        {
            \time 2/8
            c'8
            d'8
        }
        {
            \time 2/8
            e'8
            f'8
        }
        {
            \time 2/8
            g'8
            a'8
        }
    }
    '''

    measures = list(iterationtools.iterate_measures_in_expr(staff))

    assert measures[0] is staff[0]
    assert measures[1] is staff[1]
    assert measures[2] is staff[2]


def test_iterationtools_iterate_measures_in_expr_04():
    '''Optional start and stop keyword paramters.'''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    measures = list(iterationtools.iterate_measures_in_expr(staff, start=1))
    assert measures[0] is staff[1]
    assert measures[1] is staff[2]
    assert len(measures) == 2

    measures = list(iterationtools.iterate_measures_in_expr(staff, stop=2))
    assert measures[0] is staff[0]
    assert measures[1] is staff[1]
    assert len(measures) == 2



from abjad.tools import mathtools


def get_composite_offset_difference_series_from_leaves_in_expr(expr):
    r'''.. versionadded:: 2.0

    Get composite offset difference series from leaves in `expr`::

        >>> staff_1 = Staff(r"\times 4/3 { c'8 d'8 e'8 }")
        >>> staff_2 = Staff("f'8 g'8 a'8 b'8")
        >>> score = Score([staff_1, staff_2])

    ::

        >>> f(score)
            \new Score <<
                \new Staff {
                    \fraction \times 4/3 {
                        c'8
                        d'8
                        e'8
                    }
                }
                \new Staff {
                    f'8
                    g'8
                    a'8
                    b'8
                }
            >>

    ::

        >>> for x in leaftools.get_composite_offset_difference_series_from_leaves_in_expr(score):
        ...     x
        ...
        Duration(1, 8)
        Duration(1, 24)
        Duration(1, 12)
        Duration(1, 12)
        Duration(1, 24)
        Duration(1, 8)

    Composite offset difference series defined equal to time intervals between
    unique start and stop offsets of leaves in `expr`.

    Return list of durations.
    '''
    from abjad.tools import leaftools

    composite_offset_series = leaftools.get_composite_offset_series_from_leaves_in_expr(expr)

    return list(mathtools.difference_series(composite_offset_series))



def chromatic_pitch_names_string_to_named_chromatic_pitch_list(chromatic_pitch_names_string):
    '''.. versionadded:: 2.0

    Change `chromatic_pitch_names_string` to named chromatic pitch list::

        >>> string = "cs, cs cs' cs''"
        >>> result = pitchtools.chromatic_pitch_names_string_to_named_chromatic_pitch_list(string)
       
    ::

        >>> for named_chromatic_pitch in result:
        ...     named_chromatic_pitch
        ...
        NamedChromaticPitch('cs,')
        NamedChromaticPitch('cs') 
        NamedChromaticPitch("cs'") 
        NamedChromaticPitch("cs''")

    Return list of named chromatic pitches.
    '''
    from abjad.tools import pitchtools

    pitches = []
    pitch_strings = chromatic_pitch_names_string.split()
    for pitch_string in pitch_strings:
        pitch = pitchtools.NamedChromaticPitch(pitch_string)
        pitches.append(pitch)

    return pitches



import abc
from abjad.tools.pitchtools.ChromaticObject import ChromaticObject
from abjad.tools.pitchtools.PitchObject import PitchObject


class ChromaticPitchObject(PitchObject, ChromaticObject):
    '''.. versionadded:: 2.0

    Chromatic pitch base class.
    '''

    ### CLASS ATTRIBUTES ##

    __metaclass__ = abc.ABCMeta

    __slots__ = ()

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass



def list_named_chromatic_pitches_in_expr(expr):
    '''.. versionadded:: 2.0

    List named chromatic pitches in `expr`::

        >>> staff = Staff("c'4 d'4 e'4 f'4")
        >>> beam_spanner = beamtools.BeamSpanner(staff[:])

    ::

        >>> for x in pitchtools.list_named_chromatic_pitches_in_expr(beam_spanner):
        ...     x
        ...
        NamedChromaticPitch("c'")
        NamedChromaticPitch("d'")
        NamedChromaticPitch("e'")
        NamedChromaticPitch("f'")

    Return tuple.
    '''
    from abjad.tools import spannertools
    from abjad.tools import iterationtools
    from abjad.tools import leaftools
    from abjad.tools import pitchtools
    from abjad.tools import resttools

    # TODO: remove try-except
    try:
        result = pitchtools.get_named_chromatic_pitch_from_pitch_carrier(expr)
        return (result, )
    except (TypeError, MissingPitchError, ExtraPitchError):
        result = []
        if hasattr(expr, 'written_pitches'):
            result.extend(expr.written_pitches)
        # for pitch arrays
        elif hasattr(expr, 'pitches'):
            result.extend(expr.pitches)
        elif isinstance(expr, spannertools.Spanner):
            for leaf in expr.leaves:
                if hasattr(leaf, 'written_pitch') and not isinstance(leaf, resttools.Rest):
                    result.append(leaf.written_pitch)
                elif hasattr(leaf, 'written_pitches'):
                    result.extend(leaf.written_pitches)
        elif isinstance(expr, pitchtools.NamedChromaticPitchSet):
            pitches = list(expr)
            pitches.sort()
            pitches = tuple(pitches)
            return pitches
        elif isinstance(expr, (list, tuple, set)):
            for x in expr:
                result.extend(list_named_chromatic_pitches_in_expr(x))
        else:
            for leaf in iterationtools.iterate_leaves_in_expr(expr):
                if hasattr(leaf, 'written_pitch') and not isinstance(leaf, resttools.Rest):
                    result.append(leaf.written_pitch)
                elif hasattr(leaf, 'written_pitches'):
                    result.extend(leaf.written_pitches)
        return tuple(result)



from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import resttools


def replace_container_slice_with_rests(container, start=None, stop=None,
    decrease_durations_monotonically=True):
    r'''.. versionadded:: 2.10

    Replace `container` slice from `start` to `stop` with rests that decrease
    durations monotonically.

    Example 1. Replace all container elements::

        >>> container = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b' c''8")
    
    ::

        >>> container = containertools.replace_container_slice_with_rests(container)

        >>> f(container)
        \new Staff {
            r1
        }

    Example 2. Replace container elements from ``1`` forward::

        >>> container = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b' c''8")

    ::

        >>> container = containertools.replace_container_slice_with_rests(
        ...     container, start=1)

    :: 
    
        >>> f(container)
        \new Staff {
            c'8
            r2..
        }

    Example 3. Replace container elements from ``1`` to ``2``::

        >>> container = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b' c''8")

    ::
        >>> container= containertools.replace_container_slice_with_rests(
        ...         container, start=1, stop=2)

    ::

        >>> f(container)
        \new Staff {
            c'8
            r8
            e'8
            f'8
            g'8
            a'8
            b'8
            c''8
        }

    Return `container`.
    '''

    # get container elements to replace
    elements_to_replace = container[start:stop]
    
    # if there are elements to replace
    if elements_to_replace:

        # find preprolated duration of elements to replace
        duration = componenttools.sum_duration_of_components(
            elements_to_replace, preprolated=True)

        # construct rests equal in preprolated duration to replace
        rests = resttools.make_rests(duration,
            decrease_durations_monotonically=decrease_durations_monotonically)

        # replace container elements with rests
        container[start:stop] = rests

    # return container
    return container



def make_blank_line_markup():
    r'''.. versionadded:: 2.9

    Make blank line markup::

        >>> markup = markuptools.make_blank_line_markup()

    ::

        >>> markup
        Markup((MarkupCommand('fill-line', [' ']),))

    ::

        >>> f(markup)
        \markup { \fill-line { " " } }

    Return markup.
    '''
    from abjad.tools import markuptools
    
    return markuptools.Markup(r'\fill-line { " " }')



def set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(expr, key_signature=None):
    r'''.. versionadded:: 1.1

    Set ascending named diatonic pitches on nontied pitched components in `expr`::

        >>> staff = Staff(notetools.make_notes(0, [(5, 32)] * 4))
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            c'8 ~
            c'32
            d'8 ~
            d'32
            e'8 ~
            e'32
            f'8 ~
            f'32
        }

    Used primarily in generating test file examples.

    .. versionadded:: 2.0
        Optional `key_signature` keyword argument.

    Return none.

    .. versionchanged:: 2.0
        renamed ``pitchtools.diatonicize()`` to
        ``pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr()``.
    '''
    from abjad.tools import chordtools
    from abjad.tools import iterationtools
    from abjad.tools import notetools
    from abjad.tools import pitchtools
    from abjad.tools import tietools
    from abjad.tools import tonalitytools

    if key_signature is None:
        scale = tonalitytools.Scale('C', 'major')
    else:
        scale = tonalitytools.Scale(key_signature)

    dicg = scale.diatonic_interval_class_segment
    length = len(dicg)

    octave_number = 4
    pitch = pitchtools.NamedChromaticPitch(scale[0], octave_number)

    for i, tie_chain in enumerate(tietools.iterate_tie_chains_in_expr(expr)):
        if isinstance(tie_chain[0], notetools.Note):
            for note in tie_chain:
                note.written_pitch = pitch
        elif isinstance(tie_chain[0], chordtools.Chord):
            for chord in tie_chain:
                chord.written_pitches = [pitch]
        else:
            pass
        dic = dicg[i % length]
        ascending_mdi = pitchtools.MelodicDiatonicInterval(dic.quality_string, dic.number)
        pitch += ascending_mdi



from abjad import *


def test_iterationtools_iterate_notes_in_expr_01():

    staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 3)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8
            d'8
        }
        {
            \time 2/8
            e'8
            f'8
        }
        {
            \time 2/8
            g'8
            a'8
        }
    }
    '''

    generator = iterationtools.iterate_notes_in_expr(staff, reverse=True)
    notes = list(generator)

    assert notes[0] is staff[2][1]
    assert notes[1] is staff[2][0]
    assert notes[2] is staff[1][1]
    assert notes[3] is staff[1][0]
    assert notes[4] is staff[0][1]
    assert notes[5] is staff[0][0]


def test_iterationtools_iterate_notes_in_expr_02():
    '''Optional start and stop keyword parameters.'''

    staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 3)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    notes = list(iterationtools.iterate_notes_in_expr(staff, reverse=True, start=3))
    assert notes[0] is staff[1][0]
    assert notes[1] is staff[0][1]
    assert notes[2] is staff[0][0]
    assert len(notes) == 3

    notes = list(iterationtools.iterate_notes_in_expr(staff, reverse=True, start=0, stop=3))
    assert notes[0] is staff[2][1]
    assert notes[1] is staff[2][0]
    assert notes[2] is staff[1][1]
    assert len(notes) == 3

    notes = list(iterationtools.iterate_notes_in_expr(staff, reverse=True, start=2, stop=4))
    assert notes[0] is staff[1][1]
    assert notes[1] is staff[1][0]
    assert len(notes) == 2


def test_iterationtools_iterate_notes_in_expr_03():

    staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 3)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8
            d'8
        }
        {
            \time 2/8
            e'8
            f'8
        }
        {
            \time 2/8
            g'8
            a'8
        }
    }
    '''

    generator = iterationtools.iterate_notes_in_expr(staff)
    notes = list(generator)

    assert notes[0] is staff[0][0]
    assert notes[1] is staff[0][1]
    assert notes[2] is staff[1][0]
    assert notes[3] is staff[1][1]
    assert notes[4] is staff[2][0]
    assert notes[5] is staff[2][1]


def test_iterationtools_iterate_notes_in_expr_04():
    '''Optional start and stop keyword parameters.'''

    staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 3)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    notes = list(iterationtools.iterate_notes_in_expr(staff, start=3))
    assert notes[0] is staff[1][1]
    assert notes[1] is staff[2][0]
    assert notes[2] is staff[2][1]
    assert len(notes) == 3

    notes = list(iterationtools.iterate_notes_in_expr(staff, start=0, stop=3))
    assert notes[0] is staff[0][0]
    assert notes[1] is staff[0][1]
    assert notes[2] is staff[1][0]
    assert len(notes) == 3

    notes = list(iterationtools.iterate_notes_in_expr(staff, start=2, stop=4))
    assert notes[0] is staff[1][0]
    assert notes[1] is staff[1][1]
    assert len(notes) == 2



from abjad.tools import durationtools
from abjad.tools import sequencetools


# TODO: fix bug that unintentionally fractures ties.
def split_components_at_offsets(components, offsets, 
    fracture_spanners=False, cyclic=False, tie_split_notes=True, tie_split_rests=False):
    r'''.. versionadded:: 2.0
    
    Example 1. Split components cyclically and do not fracture crossing spanners::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")

    ::

        >>> beamtools.BeamSpanner(staff[0])
        BeamSpanner(|2/8(2)|)
        >>> beamtools.BeamSpanner(staff[1])
        BeamSpanner(|2/8(2)|)
        >>> spannertools.SlurSpanner(staff.leaves)
        SlurSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> componenttools.split_components_at_offsets(
        ...     staff.leaves, [Duration(3, 32)], cyclic=True)
        [[Note("c'16.")], [Note("c'32"), Note("d'16")],
        [Note("d'16"), Note("e'32")], [Note("e'16.")], [Note("f'16.")], [Note("f'32")]]

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'16. [ ( ~
                c'32
                d'16 ~
                d'16 ]
            }
            {
                e'32 [ ~
                e'16.
                f'16. ~
                f'32 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Example 2. Split components cyclically and fracture spanners::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")

    ::

        >>> beamtools.BeamSpanner(staff[0])
        BeamSpanner(|2/8(2)|)
        >>> beamtools.BeamSpanner(staff[1])
        BeamSpanner(|2/8(2)|)
        >>> spannertools.SlurSpanner(staff.leaves)
        SlurSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> result = componenttools.split_components_at_offsets(
        ... staff.leaves, [Duration(3, 32)], cyclic=True, fracture_spanners=True)

    ::

        >>> result
        [[Note("c'16.")], [Note("c'32"), Note("d'16")], [Note("d'16"), Note("e'32")],
        [Note("e'16.")], [Note("f'16.")], [Note("f'32")]]

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'16. ( ) [ ~
                c'32 (
                d'16 ) ~
                d'16 ] (
            }
            {
                e'32 ) [ ~
                e'16. (
                f'16. ) ~
                f'32 ] ( )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Example 3. Split components once and do not fracture crossing spanners::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")

    ::

        >>> beamtools.BeamSpanner(staff[0])
        BeamSpanner(|2/8(2)|)
        >>> beamtools.BeamSpanner(staff[1])
        BeamSpanner(|2/8(2)|)
        >>> spannertools.SlurSpanner(staff.leaves)
        SlurSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> offsets = [Duration(1, 32), Duration(3, 32), Duration(5, 32)]

    ::

        >>> shards = componenttools.split_components_at_offsets(
        ... staff[:1], offsets, cyclic=False, fracture_spanners=False, tie_split_notes=False)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 1/32
                c'32 [ (
            }
            {
                \time 3/32
                c'16.
            }
            {
                \time 4/32
                d'8 ]
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Example 4. Split components once and fracture crossing spanners::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")

    ::

        >>> beamtools.BeamSpanner(staff[0])
        BeamSpanner(|2/8(2)|)
        >>> beamtools.BeamSpanner(staff[1])
        BeamSpanner(|2/8(2)|)
        >>> spannertools.SlurSpanner(staff.leaves)
        SlurSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> offsets = [Duration(1, 32), Duration(3, 32), Duration(5, 32)]
        >>> shards = componenttools.split_components_at_offsets(
        ... staff[:1], offsets, cyclic=False, fracture_spanners=True, tie_split_notes=False)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 1/32
                c'32 [ ] ( )
            }
            {
                \time 3/32
                c'16. [ ] ( )
            }
            {
                \time 4/32
                d'8 [ ] (
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Example 5. Split tupletted components once and fracture crossing spanners::

        >>> staff = Staff(r"\times 2/3 { c'8 d'8 e'8 } \times 2/3 { f'8 g'8 a'8 }")

    ::

        >>> beamtools.BeamSpanner(staff[0])
        BeamSpanner({c'8, d'8, e'8})
        >>> beamtools.BeamSpanner(staff[1])
        BeamSpanner({f'8, g'8, a'8})
        >>> spannertools.SlurSpanner(staff.leaves)
        SlurSpanner(c'8, d'8, e'8, f'8, g'8, a'8)

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8 [ (
                d'8
                e'8 ]
            }
            \times 2/3 {
                f'8 [
                g'8
                a'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> offsets = [(1, 8)]
        >>> shards = componenttools.split_components_at_offsets(
        ... staff.leaves, offsets, cyclic=False, fracture_spanners=True, tie_split_notes=True)

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8 [ (
                d'16 ) ~
                d'16 (
                e'8 ]
            }
            \times 2/3 {
                f'8 [
                g'8
                a'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return list of newly split shards.

    .. note:: Add tests of tupletted notes and rests.

    .. note:: Add examples that show mark and context mark handling.

    .. note:: Add example showing grace and after grace handling.
    '''
    from abjad.tools import componenttools
    from abjad.tools import leaftools

    # check input
    assert componenttools.all_are_components(components), repr(components)
    offsets = [durationtools.Offset(offset) for offset in offsets]

    # calculate total component duration
    total_component_duration = componenttools.sum_duration_of_components(components)
    total_offset_duration = sum(offsets)

    # calculate offsets
    if cyclic:
        offsets = sequencetools.repeat_sequence_to_weight_exactly(offsets, total_component_duration)
    elif total_offset_duration < total_component_duration:
        final_offset = total_component_duration - sum(offsets)
        offsets.append(final_offset)
    elif total_component_duration < total_offset_duration:
        offsets = sequencetools.truncate_sequence_to_weight(offsets, total_component_duration)

    #print offsets, 'offsets'

    # keep copy of offsets to partition result components
    offsets_copy = offsets[:]
    
    # calculate total offset duration
    total_offset_duration = sum(offsets)
    assert total_offset_duration == total_component_duration

    # initialize loop variables
    result, shard = [], []
    offset_index, offset_count = 0, len(offsets)
    current_shard_duration = durationtools.Duration(0)
    remaining_components = list(components[:])
    advance_to_next_offset = True

    # loop and build shards by grabbing next component and next offset each time through loop
    while True:

        # grab next split point
        if advance_to_next_offset:
            if offsets:
                #print offsets
                next_split_point = offsets.pop(0)
                #print next_split_point
            else:
                #print 'no more offsets'
                break

        advance_to_next_offset = True

        # grab next component from input stack of components
        if remaining_components:
            current_component = remaining_components.pop(0)
        else:
            #print 'no more components'
            break

        # find where current component endpoint will position us
        candidate_shard_duration = current_shard_duration + current_component.prolated_duration
        #print current_component, offset_index, current_shard_duration, next_split_point

        # if current component would fill current shard exactly
        if candidate_shard_duration == next_split_point:
            #print 'exactly equal %s' % current_component
            shard.append(current_component)
            result.append(shard)
            shard = []
            current_shard_duration = durationtools.Duration(0)
            offset_index += 1
        # if current component would exceed current shard
        elif next_split_point < candidate_shard_duration:
            #print 'must split %s' % current_component

            local_split_duration = next_split_point - current_shard_duration
            #print 'local_split_duration {}'.format(local_split_duration)
            #print current_shard_duration, next_split_point, current_component, shard, local_split_duration

            if isinstance(current_component, leaftools.Leaf):
                #print 'splitting leaf ...'
                leaf_split_durations = [local_split_duration]
                additional_required_duration = current_component.prolated_duration - local_split_duration
                #print 'additional_required_duration {}'.format(additional_required_duration)
                #print 'offsets {}'.format(offsets)
                split_offsets = sequencetools.split_sequence_by_weights(
                    offsets, [additional_required_duration], cyclic=False, overhang=True)
                #print 'split_offsets {}\n'.format(split_offsets)
                additional_offsets = split_offsets[0]
                leaf_split_durations.extend(additional_offsets)
                offsets = split_offsets[-1]
                leaf_shards = leaftools.split_leaf_at_offsets(
                    current_component, leaf_split_durations, 
                    cyclic=False, fracture_spanners=fracture_spanners,tie_split_rests=tie_split_rests)
                shard.extend(leaf_shards)
                result.append(shard)
                offset_index += len(additional_offsets)
            else:
                #print 'splitting container ...'
                left_list, right_list = componenttools.split_component_at_offset(
                    current_component, local_split_duration, fracture_spanners=fracture_spanners, 
                    tie_split_notes=tie_split_notes, tie_split_rests=tie_split_rests)
                #print 'left_list, right_list {}, {}'.format(left_list, right_list)
                shard.extend(left_list)
                result.append(shard)
                # to avoid slice assignment pychecker errors
                #remaining_components[0:0] = right_list
                remaining_components.__setitem__(slice(0, 0), right_list)

            shard = []
            offset_index += 1
            current_shard_duration = durationtools.Duration(0)
        # if current component would not fill current shard
        elif candidate_shard_duration < next_split_point:
            #print 'simple append %s' % current_component
            shard.append(current_component)
            current_shard_duration += current_component.prolated_duration
            advance_to_next_offset = False
        else:
            raise ValueError
        #print ''

    # append any stub shard
    if len(shard):
        result.append(shard)

    # append any unexamined components
    if len(remaining_components):
        result.append(remaining_components)

    # group split components according to input durations
    result = sequencetools.flatten_sequence(result)
    result = componenttools.partition_components_by_durations_exactly(result, offsets_copy)

    # return list of shards
    return result



import abc

from abjad.tools.pitchtools.ObjectSet import ObjectSet


class PitchObjectSet(ObjectSet):
    '''.. versionadded:: 2.0

    Pitch set base class.
    '''

    ### CLASS METHODS ###

    __metaclass__ = abc.ABCMeta

    __slots__ = ()

    ### INITIALIZER ###

    @abc.abstractmethod
    def __new__(self):
        pass



def compute_logical_not_of_intervals(intervals):
    '''Compute the logical NOT of some collection of intervals.

    Return TimeIntervalTree.
    '''

    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return tree

    depth_tree = timeintervaltools.compute_depth_of_intervals(tree)
    logic_tree = timeintervaltools.TimeIntervalTree([x for x in depth_tree if 0 == x['depth']])

    return logic_tree



def get_context_mark_format_pieces(mark):
    '''Get context mark format pieces for `mark`.

    Return list.
    '''

    from abjad.tools import contexttools
    from abjad.tools import measuretools

    #print mark, mark.lilypond_format
    addenda = []
    mark_format = mark.lilypond_format
    if isinstance(mark_format, (tuple, list)):
        addenda.extend(mark_format)
    else:
        addenda.append(mark_format)

    # cosmetic mark is a hack to allow marks to format even without effective context;
    # currently used only in metric grid formatting
    if mark.effective_context is not None or \
        getattr(mark, '_is_cosmetic_mark', False) or \
        (isinstance(mark, contexttools.TimeSignatureMark) and
        isinstance(mark.start_component, measuretools.Measure)):
        return addenda
        #result.extend(addenda)
    else:
        addenda = [r'%%% ' + addendum + r' %%%' for addendum in addenda]
        return addenda
        #result.extend(addenda)



from abjad import *


def test_tietools_get_nontrivial_tie_chains_masked_by_components_01():
    '''Return empty list from list of untied leaves.
    '''

    t = notetools.make_repeated_notes(4)
    chains = tietools.get_nontrivial_tie_chains_masked_by_components(t)

    assert chains == []


def test_tietools_get_nontrivial_tie_chains_masked_by_components_02():
    '''Return empty list from list of untied containers.
    '''

    t = Voice(notetools.make_repeated_notes(4))
    chains = tietools.get_nontrivial_tie_chains_masked_by_components([t])

    assert chains == []


def test_tietools_get_nontrivial_tie_chains_masked_by_components_03():
    '''Returns an list of leaves on a list of tied leaves.
    '''

    t = notetools.make_repeated_notes(4)
    tietools.TieSpanner(t[0:2])
    chains = tietools.get_nontrivial_tie_chains_masked_by_components(t)

    assert chains == [tietools.TieChain(tuple(t[0:2]))]


def test_tietools_get_nontrivial_tie_chains_masked_by_components_04():
    '''Returns an list of leaves on a list of tied containers.
    '''

    t = Voice(notetools.make_repeated_notes(4))
    tietools.TieSpanner(t)
    chains = tietools.get_nontrivial_tie_chains_masked_by_components([t])

    assert chains == [tietools.TieChain(tuple(t.leaves))]


def test_tietools_get_nontrivial_tie_chains_masked_by_components_05():
    '''Returns an list of two elements if two Tie spanners are found.
    '''

    t = Voice("c'8 d'8 e'8 f'8")
    tietools.TieSpanner(t[:2])
    tietools.TieSpanner(t[2:])
    chains = tietools.get_nontrivial_tie_chains_masked_by_components(t.leaves)

    assert chains == [tietools.TieChain(tuple(t[0:2])), tietools.TieChain(tuple(t[2:]))]


def test_tietools_get_nontrivial_tie_chains_masked_by_components_06():
    '''returns an empty list if the given list of components is not
    tie-spanned, while its decendents are.
    '''

    t = Voice("c'8 d'8 e'8 f'8")
    tietools.TieSpanner(t[0:2])
    tietools.TieSpanner(t[2:])
    chains = tietools.get_nontrivial_tie_chains_masked_by_components([t])

    assert chains == []


def test_tietools_get_nontrivial_tie_chains_masked_by_components_07():
    '''returns an list those leaves that intersect a Tie spanner and the
    components given.
    '''

    t = Voice("c'8 d'8 e'8 f'8")
    tietools.TieSpanner(t.leaves)
    chains = tietools.get_nontrivial_tie_chains_masked_by_components(t.leaves[1:3])

    assert chains == [tietools.TieChain(tuple(t.leaves[1:3]))]


def test_tietools_get_nontrivial_tie_chains_masked_by_components_08():
    '''Works across containers.
    '''

    t = Voice(Container("c'8 d'8 e'8 f'8") * 3)
    tietools.TieSpanner(t[0:2])
    chains = tietools.get_nontrivial_tie_chains_masked_by_components(t[:])

    assert chains == [tietools.TieChain(tuple(t.leaves[0:8]))]



import abc
from abjad.tools.pitchtools.CounterpointObject import CounterpointObject
from abjad.tools.pitchtools.IntervalObject import IntervalObject


class CounterpointIntervalObject(IntervalObject, CounterpointObject):
    '''..versionadded:: 2.0

    Counterpoint interval base class.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __abs__(self):
        return type(self)(abs(self._number))

    def __float__(self):
        return float(self._number)

    def __int__(self):
        return self._number

    ### PUBLIC PROPERTIES ###

    @property
    def number(self):
        return self._number

    @property
    def semitones(self):
        raise NotImplementedError



def compute_logical_or_of_intervals_in_interval(intervals, interval):
    '''Compute the logical OR of a collection of intervals,
    cropped within `interval`.

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return tree

    depth_tree = timeintervaltools.compute_depth_of_intervals_in_interval(tree, interval)
    logic_tree = timeintervaltools.TimeIntervalTree([x for x in depth_tree if 1 <= x['depth']])

    return logic_tree



from abjad.tools import iterationtools
from abjad.tools import tuplettools


def apply_multipart_beam_spanner_to_bottommost_tuplets_in_expr(expr):
    r'''Beam bottommost tuplets in `expr`::

        >>> staff = Staff(3 * Tuplet(Fraction(2, 3), "c'8 d'8 e'8"))

    ::

        f(staff)
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            \times 2/3 {
                c'8
                d'8
                e'8
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> beamtools.apply_multipart_beam_spanner_to_bottommost_tuplets_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8 [
                d'8
                e'8 ]
            }
            \times 2/3 {
                c'8 [
                d'8
                e'8 ]
            }
            \times 2/3 {
                c'8 [
                d'8
                e'8 ]
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return none.

    .. versionchanged:: 2.9
        renamed ``tuplettools.beam_bottommost_tuplets_in_expr()`` to
        ``beamtools.apply_multipart_beam_spanner_to_bottommost_tuplets_in_expr()``.

    .. versionchanged:: 2.9
        renamed ``beamtools.beam_bottommost_tuplets_in_expr()`` to
        ``beamtools.apply_multipart_beam_spanner_to_bottommost_tuplets_in_expr()``.
    '''
    from abjad.tools import beamtools

    for tuplet in iterationtools.iterate_components_in_expr(expr, tuplettools.Tuplet):
        for component in tuplet:
            if isinstance(component, tuplettools.Tuplet):
                break
        else:
            beamtools.MultipartBeamSpanner(tuplet)



import copy
from abjad import *


def apply_bowing_marks(score):

    # apply alternating upbow and downbow for first two sounding bars
    # of the first violin
    for measure in score['First Violin Voice'][6:8]:
        for i, chord in enumerate(iterationtools.iterate_chords_in_expr(measure)):
            if i % 2 == 0:
                marktools.Articulation('downbow')(chord)
            else:
                marktools.Articulation('upbow')(chord)

    # create and apply rebowing markup
    rebow_markup = markuptools.Markup(
        markuptools.MarkupCommand(
            'concat', [
                markuptools.MusicGlyph('scripts.downbow'),
                markuptools.MarkupCommand('hspace', 1),
                markuptools.MusicGlyph('scripts.upbow'),
            ]))
    copy.copy(rebow_markup)(score['First Violin Voice'][64][0]) 
    copy.copy(rebow_markup)(score['Second Violin Voice'][75][0]) 
    copy.copy(rebow_markup)(score['Viola Voice'][86][0]) 




from abjad import *


def test_marktools_get_marks_attached_to_components_in_expr_01():

    staff = Staff(r"c'4 \pp d' \staccato e' \ff f' \staccato")

    assert len(marktools.get_marks_attached_to_components_in_expr(staff)) == 4



from abjad.tools import documentationtools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import tietools
from abjad.tools import tuplettools
from abjad.tools.datastructuretools.TreeContainer import TreeContainer
from abjad.tools.rhythmtreetools.RhythmTreeNode import RhythmTreeNode


class RhythmTreeContainer(RhythmTreeNode, TreeContainer):
    r'''A container node in a rhythm tree structure:

    ::

        >>> container = rhythmtreetools.RhythmTreeContainer(duration=1, children=[])
        >>> container
        RhythmTreeContainer(
            duration=Duration(1, 1)
            )

    Similar to Abjad containers, `RhythmTreeContainer` supports a list interface,
    and can be appended, extended, indexed and so forth by other `RhythmTreeNode`
    subclasses:

    ::

        >>> leaf_a = rhythmtreetools.RhythmTreeLeaf(duration=1)
        >>> leaf_b = rhythmtreetools.RhythmTreeLeaf(duration=2)
        >>> container.extend([leaf_a, leaf_b])
        >>> container
        RhythmTreeContainer(
            children=(
                RhythmTreeLeaf(
                    duration=Duration(1, 1),
                    is_pitched=True
                    ),
                RhythmTreeLeaf(
                    duration=Duration(2, 1),
                    is_pitched=True
                    )
                ),
            duration=Duration(1, 1)
            )

    ::

        >>> another_container = rhythmtreetools.RhythmTreeContainer(duration=2)
        >>> another_container.append(rhythmtreetools.RhythmTreeLeaf(duration=3))
        >>> another_container.append(container[1])
        >>> container.append(another_container)
        >>> container
        RhythmTreeContainer(
            children=(
                RhythmTreeLeaf(
                    duration=Duration(1, 1),
                    is_pitched=True
                    ),
                RhythmTreeContainer(
                    children=(
                        RhythmTreeLeaf(
                            duration=Duration(3, 1),
                            is_pitched=True
                            ),
                        RhythmTreeLeaf(
                            duration=Duration(2, 1),
                            is_pitched=True
                            )
                        ),
                    duration=Duration(2, 1)
                    )
                ),
            duration=Duration(1, 1)
            )

    Call `RhythmTreeContainer` with a duration to generate a tuplet structure:

    ::

        >>> container((1, 4))
        [FixedDurationTuplet(1/4, [c'8, {@ 5:4 c'8., c'8 @}])]

    ::

        >>> f(_[0])
        \times 2/3 {
            c'8
            \times 4/5 {
                c'8.
                c'8
            }
        }

    Returns `RhythmTreeContainer` instance.
    '''

    ### INITIALIZER ###

    def __init__(self, children=None, duration=1, name=None):
        RhythmTreeNode.__init__(self, duration=duration, name=name)
        self._children = []
        if isinstance(children, type(None)):
            pass
        elif isinstance(children, (list, str, tuple)):
            self.extend(children)
        else:
            raise ValueError('Cannot instantiate {} with {!r}.'.format(type(self), children))

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        '''Concatenate containers self and expr. The operation c = a + b
        returns a new RhythmTreeContainer c with the content of both a and b,
        and a duration equal to the sum of the durations of a and b. The
        operation is non-commutative: the content of the first operand will be
        placed before the content of the second operand:

        ::

            >>> a = rhythmtreetools.RhythmTreeParser()('(1 (1 1 1))')[0]
            >>> b = rhythmtreetools.RhythmTreeParser()('(2 (3 4))')[0]

        ::

            >>> c = a + b

        ::

            >>> c.duration
            Duration(3, 1)

        ::

            >>> c
            RhythmTreeContainer(
                children=(
                    RhythmTreeLeaf(
                        duration=Duration(1, 1),
                        is_pitched=True
                        ),
                    RhythmTreeLeaf(
                        duration=Duration(1, 1),
                        is_pitched=True
                        ),
                    RhythmTreeLeaf(
                        duration=Duration(1, 1),
                        is_pitched=True
                        ),
                    RhythmTreeLeaf(
                        duration=Duration(3, 1),
                        is_pitched=True
                        ),
                    RhythmTreeLeaf(
                        duration=Duration(4, 1),
                        is_pitched=True
                        )
                    ),
                duration=Duration(3, 1)
                )

        Return new RhythmTreeContainer.
        '''
        from abjad.tools.rhythmtreetools.RhythmTreeParser import RhythmTreeParser
        if isinstance(expr, str):
            expr = RhythmTreeParser()(expr)
            assert 1 == len(expr) and isinstance(expr[0], type(self))
            expr = expr[0]
        container = type(self)(duration=self.duration + expr.duration)
        container.extend(self[:])
        container.extend(expr[:])
        return container

    def __call__(self, pulse_duration):
        '''Generate Abjad score components:

        ::

            >>> rtm = '(1 (1 (2 (1 1 1)) 2))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]

        ::

            >>> tree((1, 4))
            [FixedDurationTuplet(1/4, [c'16, {@ 3:2 c'16, c'16, c'16 @}, c'8])]

        Return sequence of components.
        '''
        pulse_duration = durationtools.Duration(pulse_duration)
        assert 0 < pulse_duration
        def recurse(node, tuplet_duration):
            basic_prolated_duration = tuplet_duration / node.contents_duration
            basic_written_duration = basic_prolated_duration.equal_or_greater_power_of_two
            tuplet = tuplettools.FixedDurationTuplet(tuplet_duration, [])
            for child in node.children:
                if isinstance(child, type(self)):
                    tuplet.extend(recurse(child, child.duration * basic_written_duration))
                else:
                    leaves = child(basic_written_duration)
                    tuplet.extend(leaves)
                    if 1 < len(leaves):
                        tietools.TieSpanner(leaves)
            if tuplet.multiplier == 1:
                return tuplet[:]
            return [tuplet]
        result = recurse(self, pulse_duration * self.duration)
        tuplettools.remove_trivial_tuplets_in_expr(result)
        return result

    def __eq__(self, other):
        '''True if type, duration and children are equivalent, otherwise False.

        Return boolean.
        '''
        if type(self) == type(other):
            if self.duration == other.duration:
                if self.children == other.children:
                    return True
        return False

    def __setitem__(self, i, expr):
        '''Set `expr` in self at nonnegative integer index `i`, or set `expr` in self at slice i.
        Replace contents of `self[i]` with `expr`.
        Attach parentage to contents of `expr`, and detach parentage of any replaced nodes.

        ::

            >>> a = rhythmtreetools.RhythmTreeContainer()
            >>> b = rhythmtreetools.RhythmTreeLeaf()
            >>> c = rhythmtreetools.RhythmTreeLeaf()

        ::

            >>> a.append(b)
            >>> b.parent is a
            True

        ::

            >>> a.children == (b,)
            True

        ::

            >>> a[0] = c

        ::

            >>> c.parent is a
            True

        ::

            >>> b.parent is None
            True

        ::

            >>> a.children == (c,)
            True

        Return `None`.
        '''
        from abjad.tools.rhythmtreetools.RhythmTreeParser import RhythmTreeParser

        proper_parentage = self.proper_parentage

        if isinstance(i, int):        
            if isinstance(expr, str):
                expr = RhythmTreeParser()(expr)[0]
                assert len(expr) == 1
                expr = expr[0]
            else:
                assert isinstance(expr, self._node_klass)
            old = self[i]
            assert expr not in proper_parentage
            old._switch_parent(None)
            expr._switch_parent(self)
            self._children.insert(i, expr)
        else:
            if isinstance(expr, str):
                expr = RhythmTreeParser()(expr)
            elif isinstance(expr, list) and len(expr) == 1 and isinstance(expr[0], str):
                expr = RhythmTreeParser()(expr[0])
            else:
                assert all([isinstance(x, self._node_klass) for x in expr])
            if i.start == i.stop and i.start is not None \
                and i.stop is not None and i.start <= -len(self):
                start, stop = 0, 0
            else:
                start, stop, stride = i.indices(len(self))
            old = self[start:stop]
            for node in expr:
                assert node not in proper_parentage
            for node in old:
                node._switch_parent(None)
            for node in expr:
                node._switch_parent(self)
            self._children.__setitem__(slice(start, start), expr)
        self._mark_entire_tree_for_later_update()

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _leaf_klass(self):
        from abjad.tools import rhythmtreetools
        return rhythmtreetools.RhythmTreeLeaf

    @property
    def _node_klass(self):
        return RhythmTreeNode

    @property
    def _pretty_rtm_format_pieces(self):
        result = []
        result.append('({} ('.format(self.duration))
        for child in self:
            result.extend(['\t' + x for x in child._pretty_rtm_format_pieces])
        result[-1] = result[-1] + '))'
        return result

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def contents_duration(self):
        '''The total duration of the children of a `RhythmTreeContainer` instance:

        ::

            >>> rtm = '(1 (1 (2 (1 1 1)) 2))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]

        ::

            >>> tree.contents_duration
            Duration(5, 1)

        ::

            >>> tree[1].contents_duration
            Duration(3, 1)

        Return int.
        '''
        return sum([x.duration for x in self])

    @property
    def graphviz_graph(self):
        '''The GraphvizGraph representation of the RhythmTreeContainer:

        ::

            >>> rtm = '(1 (1 (2 (1 1 1)) 2))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
            >>> graph = tree.graphviz_graph
            >>> print graph.graphviz_format
            digraph G {
                node_0 [label=1,
                    shape=triangle];
                node_1 [label=1,
                    shape=box];
                node_2 [label=2,
                    shape=triangle];
                node_3 [label=1,
                    shape=box];
                node_4 [label=1,
                    shape=box];
                node_5 [label=1,
                    shape=box];
                node_6 [label=2,
                    shape=box];
                node_0 -> node_1;
                node_0 -> node_2;
                node_0 -> node_6;
                node_2 -> node_3;
                node_2 -> node_4;
                node_2 -> node_5;
            }

        ::

            >>> iotools.graph(graph) # doctest: +SKIP

        Return `GraphvizGraph` instance.
        '''

        graph = documentationtools.GraphvizGraph(name='G')
        node_mapping = {}
        for node in self.nodes:
            graphviz_node = documentationtools.GraphvizNode()
            graphviz_node.attributes['label'] = str(node.duration)
            if isinstance(node, type(self)):
                graphviz_node.attributes['shape'] = 'triangle'   
            else:
                graphviz_node.attributes['shape'] = 'box'
            graph.append(graphviz_node)
            node_mapping[node] = graphviz_node
            if node.parent is not None:
                documentationtools.GraphvizEdge()(
                    node_mapping[node.parent],
                    node_mapping[node],
                    )
        return graph

    @property
    def rtm_format(self):
        '''The node's RTM format:

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
            >>> tree.rtm_format
            '(1 ((1 (1 1)) (1 (1 1))))'

        Return string.
        '''
        return '({} ({}))'.format(
            self.duration,
            ' '.join([x.rtm_format for x in self]))





from abjad.tools import tuplettools


def iterate_tuplets_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.10

    Iterate tuplets forward in `expr`::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
        >>> Tuplet(Fraction(2, 3), staff[:3])
        Tuplet(2/3, [c'8, d'8, e'8])
        >>> Tuplet(Fraction(2, 3), staff[-3:])
        Tuplet(2/3, [a'8, b'8, c''8])

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                e'8
            }
            f'8
            g'8
            \times 2/3 {
                a'8
                b'8
                c''8
            }
        }

        >>> for tuplet in iterationtools.iterate_tuplets_in_expr(staff):
        ...     tuplet
        ...
        Tuplet(2/3, [c'8, d'8, e'8])
        Tuplet(2/3, [a'8, b'8, c''8])

    Iterate tuplets backward in `expr`::

        >>> for tuplet in iterationtools.iterate_tuplets_in_expr(staff, reverse=True):
        ...     tuplet
        ...
        Tuplet(2/3, [a'8, b'8, c''8])
        Tuplet(2/3, [c'8, d'8, e'8])

    Ignore threads.

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, klass=tuplettools.Tuplet, reverse=reverse, start=start, stop=stop)



from abjad.tools import componenttools
from abjad.tools import leaftools


def iterate_timeline_from_component(expr, klass=None, reverse=False):
    r'''.. versionadded:: 2.10

    Iterate timeline forward from `component`::

        >>> score = Score([])
        >>> score.append(Staff(notetools.make_repeated_notes(4, Duration(1, 4))))
        >>> score.append(Staff(notetools.make_repeated_notes(4)))
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(score)

    ::

        >>> f(score)
        \new Score <<
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }
            \new Staff {
                g'8
                a'8
                b'8
                c''8
            }
        >>

    ::

        >>> for leaf in iterationtools.iterate_timeline_from_component(score[1][2]):
        ...     leaf
        ...
        Note("b'8")
        Note("c''8")
        Note("e'4")
        Note("f'4")

    Iterate timeline backward from `component`::

    ::

        >>> for leaf in iterationtools.iterate_timeline_from_component(score[1][2], reverse=True):
        ...     leaf
        ...
        Note("b'8")
        Note("c'4")
        Note("a'8")
        Note("g'8")

    Yield components sorted backward by score offset stop time when `reverse` is True.

    Iterate leaves when `klass` is none.

    .. todo:: optimize to avoid behind-the-scenes full-score traversal.
    '''
    from abjad.tools import iterationtools

    if klass is None:
        klass = leaftools.Leaf

    component_generator = iterationtools.iterate_timeline_in_expr(
        expr.parentage.root, klass=klass, reverse=reverse)

    yielded_expr = False
    for component in component_generator:
        if yielded_expr:
            yield component
        elif component is expr:
            yield component
            yielded_expr = True



from abjad.tools.quantizationtools.QEvent import QEvent


class TerminalQEvent(QEvent):
    '''The terminal event in a series of ``QEvents``:

        >>> q_event = quantizationtools.TerminalQEvent(1000)
        >>> q_event
        quantizationtools.TerminalQEvent(
            durationtools.Offset(1000, 1)
            )

    Carries no significance outside the context of a ``QEventSequence``.

    Return ``TerminalQEvent`` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_offset',)

    ### INITIALIZER ###

    def __init__(self, offset):
        QEvent.__init__(self, offset)

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        if type(self) == type(other) and \
            self.offset == other.offset:
            return True
        return False



from abjad.tools import durationtools
from abjad.tools.abctools import AbjadObject


class MetricalKernel(AbjadObject):
    '''A metrical kernel, or offset-impulse-response-filter:

    ::

        >>> hierarchy = timesignaturetools.MetricalHierarchy((5, 8))
        >>> kernel = hierarchy.generate_offset_kernel_to_denominator(8)
        >>> kernel
        MetricalKernel({
            Offset(0, 1): Multiplier(3, 11),
            Offset(1, 8): Multiplier(1, 11),
            Offset(1, 4): Multiplier(1, 11),
            Offset(3, 8): Multiplier(2, 11),
            Offset(1, 2): Multiplier(1, 11),
            Offset(5, 8): Multiplier(3, 11)
        })
  
    Call the kernel against an expression from which offsets can be counted
    to receive an impulse-response:

    ::

        >>> offsets = [(0, 8), (1, 8), (1, 8), (3, 8)]
        >>> kernel(offsets)
        0.6363636363636364

    Return `MetricalKernel` instance.
    '''
    ### CLASS ATTRIBUTES ###

    __slots__ = ('_kernel', '_offsets')

    ### INITIALIZER ###

    def __init__(self, kernel):
        assert isinstance(kernel, dict)
        assert 1 < len(kernel)
        for key, value in kernel.iteritems():
            assert isinstance(key, durationtools.Offset)
            assert isinstance(value, durationtools.Multiplier)
        self._kernel = kernel.copy()
        self._offsets = tuple(sorted(self._kernel))

    ### SPECIAL METHODS ###

    def __call__(self, expr):
        offset_count = durationtools.count_offsets_in_expr(expr)
        response = 0.
        for offset, count in offset_count.iteritems():
            if offset in self._kernel:
                response += (self._kernel[offset] * count)
        return float(response)

    def __eq__(self, other):
        if type(self) == type(other):
            if self._kernel == other._kernel:
                return True
        return False

    def __repr__(self):
        result = ['{}({{'.format(self._class_name)]
        offsets = sorted(self._kernel)
        for offset in offsets[:-1]:
            result.append('\t{!r}: {!r},'.format(offset, self._kernel[offset]))
        result.append('\t{!r}: {!r}'.format(offsets[-1], self._kernel[offsets[-1]]))
        result.append('})')
        return '\n'.join(result)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def kernel(self):
        '''The kernel datastructure.

        Return dict.
        '''
        return self._kernel.copy()




def list_known_markup_functions():
    '''List all markup functions recognized by ``LilyPondParser``:

    ::

        >>> for x in lilypondparsertools.list_known_markup_functions():
        ...     print x
        ...
        abs-fontsize
        arrow-head
        auto-footnote
        backslashed-digit
        beam
        bold
        box
        bracket
        caps
        center-align
        center-column
        char
        circle
        column
        column-lines
        combine
        concat
        customTabClef
        dir-column
        doubleflat
        doublesharp
        draw-circle
        draw-hline
        draw-line
        dynamic
        epsfile
        eyeglasses
        fill-line
        fill-with-pattern
        filled-box
        finger
        flat
        fontCaps
        fontsize
        footnote
        fraction
        fret-diagram
        fret-diagram-terse
        fret-diagram-verbose
        fromproperty
        general-align
        halign
        harp-pedal
        hbracket
        hcenter-in
        hspace
        huge
        italic
        justified-lines
        justify
        justify-field
        justify-string
        large
        larger
        left-align
        left-brace
        left-column
        line
        lookup
        lower
        magnify
        markalphabet
        markletter
        medium
        musicglyph
        natural
        normal-size-sub
        normal-size-super
        normal-text
        normalsize
        note
        note-by-number
        null
        number
        on-the-fly
        override
        override-lines
        pad
        pad-around
        pad-to-box
        pad-x
        page-link
        page-ref
        parenthesize
        path
        pattern
        postscript
        property-recursive
        put-adjacent
        raise
        replace
        rest
        rest-by-number
        right-align
        right-brace
        right-column
        roman
        rotate
        rounded-box
        sans
        scale
        score
        semiflat
        semisharp
        sesquiflat
        sesquisharp
        sharp
        simple
        slashed-digit
        small
        smallCaps
        smaller
        stencil
        strut
        sub
        super
        table-of-contents
        teeny
        text
        tied-lyric
        tiny
        translate
        translate-scaled
        transparent
        triangle
        typewriter
        underline
        upright
        vcenter
        verbatim-file
        vspace
        whiteout
        with-color
        with-dimensions
        with-link
        with-url
        woodwind-diagram
        wordwrap
        wordwrap-field
        wordwrap-internal
        wordwrap-lines
        wordwrap-string
        wordwrap-string-internal

    Return list.
    '''
    from abjad.ly import markup_functions
    from abjad.ly import markup_list_functions

    return sorted(markup_functions.keys() + markup_list_functions.keys())



from abjad.tools import componenttools


def remove_leafless_containers_in_expr(expr):
    r'''Remove empty containers in `expr`::

        >>> staff = Staff("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 } { b'8 c''8 }")
        >>> beamtools.BeamSpanner(staff[:])
        BeamSpanner({c'8, d'8}, {e'8, f'8}, {g'8, a'8}, {b'8, c''8})

    ::

        >>> containertools.delete_contents_of_container(staff[1])
        Selection(Note("e'8"), Note("f'8"))
        >>> containertools.delete_contents_of_container(staff[-1])
        Selection(Note("b'8"), Note("c''8"))

    ::

        >>> f(staff)
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
            }
            {
                g'8
                a'8 ]
            }
            {
            }
        }

    ::

        >>> containertools.remove_leafless_containers_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                g'8
                a'8 ]
            }
        }

    Return none.

    .. versionchanged:: 2.0
        renamed ``containertools.remove_empty()`` to
        ``containertools.remove_leafless_containers_in_expr()``.
    '''
    from abjad.tools import iterationtools

    for container in iterationtools.iterate_containers_in_expr(expr):
        if not container.leaves:
            componenttools.remove_component_subtree_from_score_and_spanners([container])



import abc
import inspect
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadObject


class QEvent(AbjadObject):
    '''Abstract base class from which concrete ``QEvent`` subclasses inherit.

    Represents an attack point to be quantized.

    All ``QEvents`` possess a rational offset in milliseconds, and an optional
    index for disambiguating events which fall on the same offset in a ``QGrid``.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_index', '_offset')

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, offset, index=None):
        offset = durationtools.Offset(offset)
        self._offset = offset
        self._index = index

    ### SPECIAL METHODS ###

    def __getstate__(self):
        state = {}
        for klass in inspect.getmro(self.__class__):
            if hasattr(klass, '__slots__'):
                for slot in klass.__slots__:
                    if slot not in state:
                        state[slot] = getattr(self, slot)
        return state

    def __lt__(self, other):
        if type(self) == type(self):
            if self.offset < other.offset:
                return True
        return False

    def __repr__(self):
        return '\n'.join(self._get_tools_package_qualified_repr_pieces())

    def __setstate__(self, state):
        for key, value in state.iteritems():
            setattr(self, key, value)

    ### PRIVATE PROPERTIES ###

    @property
    def index(self):
        '''The optional index, for sorting QEvents with identical offsets.'''
        return self._index

    @property
    def offset(self):
        '''The offset in milliseconds of the event.'''
        return self._offset



import abc
import collections
from abjad.tools.abctools import ImmutableAbjadObject


class QSchemaItem(tuple, ImmutableAbjadObject):
    '''`QSchemaItem` represents a change of state in the timeline of a quantization process.

    `QSchemaItem` is abstract and immutable.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    ### INITIALIZER ###

    @abc.abstractmethod
    def __new__(klass):
        raise Exception

    ### SPECIAL METHODS ###

    def __getnewargs__(self):
        'Return self as a plain tuple.  Used by copy and pickle.'
        return tuple(self)

    def __repr__(self):
        pieces = self._get_tools_package_qualified_keyword_argument_repr_pieces()
        if pieces:
            result = ['{}('.format(self._class_name)]
            result.extend(self._get_tools_package_qualified_keyword_argument_repr_pieces())
            result.append('\t)')
            return '\n'.join(result)
        return '{}()'.format(self._class_name)

    ### SPECIAL PROPERTIES ###

    @property
    def __dict__(self):
        return collections.OrderedDict(zip(self._fields, self))




def offset_happens_before_timespan_stops(timespan=None, offset=None, hold=False):
    r'''.. versionadded:: 2.11

    Make time relation indicating that `offset` happens before `timespan` stops::

        >>> timerelationtools.offset_happens_before_timespan_stops()
        OffsetTimespanTimeRelation('offset < timespan.stop')

    '''
    from abjad.tools import timerelationtools

    time_relation = timerelationtools.OffsetTimespanTimeRelation(
        'offset < timespan.stop',
        timespan=timespan, offset=offset)

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



from abjad.tools import iterationtools


def apply_complex_beam_spanners_to_measures_in_expr(expr):
    r'''.. versionadded:: 2.0

    Apply complex beam spanners to measures in `expr`::

        >>> staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> beamtools.apply_complex_beam_spanners_to_measures_in_expr(staff)
        [ComplexBeamSpanner(|2/8(2)|), ComplexBeamSpanner(|2/8(2)|)]

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                \set stemLeftBeamCount = #0
                \set stemRightBeamCount = #1
                c'8 [
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #0
                d'8 ]
            }
            {
                \set stemLeftBeamCount = #0
                \set stemRightBeamCount = #1
                e'8 [
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #0
                f'8 ]
            }
        }


    ::

        >>> show(staff) # doctest: +SKIP

    Return list of beams created.

    .. versionchanged:: 2.9
        renamed ``measuretools.apply_complex_beam_spanners_to_measures_in_expr()`` to
        ``beamtools.apply_complex_beam_spanners_to_measures_in_expr()``.
    '''
    from abjad.tools import beamtools

    # init beams created
    beams_created = []

    # apply complex beam spanners to measures in expr
    for measure in iterationtools.iterate_measures_in_expr(expr):
        beam = beamtools.ComplexBeamSpanner(measure)
        beams_created.append(beam)

    # return beams created
    return beams_created



def detach_marks_attached_to_components_in_expr(expr):
    r'''.. versionadded:: 2.9

    Detach marks attached to components in `expr`::

        >>> staff = Staff("c'4 \staccato d' \marcato e' \staccato f' \marcato")

    ::

        >>> f(staff)
        \new Staff {
            c'4 -\staccato
            d'4 -\marcato
            e'4 -\staccato
            f'4 -\marcato
        }

    ::

        >>> for mark in marktools.detach_marks_attached_to_components_in_expr(staff):
        ...     mark
        ...
        Articulation('staccato')
        Articulation('marcato')
        Articulation('staccato')
        Articulation('marcato')

    ::

        >>> f(staff)
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }

    Return tuple of zero or more detached marks.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import marktools

    result = []
    for component in iterationtools.iterate_components_in_expr(expr):
        result.extend(marktools.detach_marks_attached_to_component(component))

    return tuple(result)



from abjad.tools import spannertools


def get_tie_chain(component):
    r'''.. versionadded:: 2.0

    Get tie chain from `component`::

        >>> staff = Staff("c'8 ~ c' d'4")

    ::

        >>> f(staff)
        \new Staff {
            c'8 ~
            c'8
            d'4
        }

    ::

        >>> tietools.get_tie_chain(staff[0])
        TieChain(Note("c'8"), Note("c'8"))

    Return tie chain.
    '''
    from abjad.tools import tietools


    tie_spanners = spannertools.get_spanners_attached_to_component(component, tietools.TieSpanner)
    count = len(tie_spanners)

    if count == 0:
        return tietools.TieChain(music=component)
    elif count == 1:
        return tietools.TieChain(music=tie_spanners.pop().leaves)
    else:
        raise ExtraSpannerError



from abjad.tools import componenttools


def fuse_tied_leaves_in_components_once_by_durations_without_overhang(components, durations):
    r'''.. versionadded:: 1.1

    Fuse tied leaves in `components` once by prolated `durations` without overhang::

        >>> staff = Staff(notetools.make_repeated_notes(8))
        >>> tietools.TieSpanner(staff.leaves)
        TieSpanner(c'8, c'8, c'8, c'8, c'8, c'8, c'8, c'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8 ~
            c'8 ~
            c'8 ~
            c'8 ~
            c'8 ~
            c'8 ~
            c'8 ~
            c'8
        }

    ::

        >>> leaftools.fuse_tied_leaves_in_components_once_by_durations_without_overhang(
        ... staff, [Duration(3, 8), Duration(3, 8)])

    ::

        >>> f(staff)
        \new Staff {
            c'4. ~
            c'4. ~
            c'8 ~
            c'8
        }

    Return none.

    .. versionchanged:: 2.0
        renamed ``fuse.tied_leaves_by_prolated_durations()`` to
        ``leaftools.fuse_tied_leaves_in_components_once_by_durations_without_overhang()``.
    '''
    from abjad.tools import leaftools
    from abjad.tools import tietools

    # get duration groups
    groups = componenttools.partition_components_by_durations_exactly(
        components, durations, cyclic=False, in_seconds=False, overhang=False)

    for group in groups:
        # get tie chains intersecting this group
        tie_chains = tietools.get_nontrivial_tie_chains_masked_by_components(group)
        for tie_chain in tie_chains:
            leaftools.fuse_leaves_in_tie_chain_by_immediate_parent(tie_chain)



from abjad.tools import abctools
import collections
import os
import time


class AbjadConfig(collections.MutableMapping, abctools.AbjadObject):
    '''Abjad configuration object:

    ::

        >>> from abjad.tools.configurationtools import AbjadConfig
        >>> ABJCONFIG = AbjadConfig()
        >>> ABJCONFIG['accidental_spelling']
        'mixed'

    On instantiation, `AbjadConfig` creates the `$HOME/.abjad/` directory
    if it does not already exist.

    It then attempts to read an `abjad.cfg` file in that directory,
    parsing it as a `ConfigObj` configuration.  A default configuration
    is generated if no file is found.

    The `ConfigObj` instance is validated, and key:value pairs which
    fail validation are replaced by default values.

    The configuration is then written back to disk.

    Finally, the Abjad output directory is created if it does not already exist,
    by referencing the 'abjad_output' key in the configuration.

    `AbjadConfig` supports the mutable mapping interface, and can be subscripted as a dictionary.

    Returns `AbjadConfig` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_settings')

    ### INITIALIZER ###

    def __init__(self):

        import configobj
        import validate

        # verify configuration directory
        if not os.path.exists(self.ABJAD_CONFIG_DIRECTORY_PATH):
            os.mkdir(self.ABJAD_CONFIG_DIRECTORY_PATH)
        
        # attempt to load config from disk, and validate
        # a config object will be created if none is found on disk
        spec = self.get_config_spec()
        config = configobj.ConfigObj(self.ABJAD_CONFIG_FILE_PATH, configspec=spec)

        # validate
        validator = validate.Validator()
        validation = config.validate(validator, copy=True)

        # replace failing key:value pairs with default values
        if validation is not True:
            for key, valid in validation.iteritems():
                if not valid:
                    default = config.default_values[key]
                    print 'Warning: Abjad config key {!r} failed validation, '\
                        'setting to default: {!r}.'.format(key, default)
                    config[key] = default

        # setup output formatting
        config.write_empty_values = True
        config.comments.update(self.get_option_comments())
        config.initial_comment = self.get_initial_comment()

        # write back to disk
        with open(self.ABJAD_CONFIG_FILE_PATH, 'w') as f:
            config.write(f)

        # turn the ConfigObj instance into a standard dict,
        # and replace its empty string values with Nones,
        # caching the result on this AbjadConfig instance.
        self._settings = dict(config)
        for key, value in self._settings.iteritems():
            if value == '' or value == 'None':
                self._settings[key] = None

        # finally, verify the PDF output directory
        if not os.path.exists(self.ABJAD_OUTPUT_PATH):
            os.mkdir(self.ABJAD_OUTPUT_PATH)

    ### SPECIAL METHODS ###

    def __delitem__(self, i):
        del(self._settings[i])

    def __getitem__(self, i):
        return self._settings[i]

    def __iter__(self):
        for key in self._settings:
            yield key

    def __len__(self):
        return len(self._settings)

    def __setitem__(self, i, arg):
        self._settings[i] = arg

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def ABJAD_CONFIG_DIRECTORY_PATH(self):
        return os.path.join(self.HOME_PATH, '.abjad')

    @property
    def ABJAD_CONFIG_FILE_PATH(self):
        return os.path.join(self.ABJAD_CONFIG_DIRECTORY_PATH, 'abjad.cfg')

    @property
    def ABJAD_EXPERIMENTAL_PATH(self):
        return os.path.abspath(os.path.join(self.ABJAD_PATH, '..', '..', 'experimental'))

    @property
    def ABJAD_OUTPUT_PATH(self):
        return self._settings['abjad_output']

    @property
    def ABJAD_PATH(self):
        module_parts = self.__module__.split('.')
        filepath_parts = os.path.abspath(__file__).rpartition('.py')[0].split(os.path.sep)
        for part in reversed(module_parts):
            if part == 'abjad':
                break
            filepath_parts.pop()
        return os.path.sep.join(filepath_parts)

    @property
    def ABJAD_ROOT_PATH(self):
        return os.path.abspath(os.path.join(self.ABJAD_PATH, '..', '..'))

    @property
    def HOME_PATH(self):
        return os.environ.get('HOME') or os.environ.get('HOMEPATH') or os.environ.get('APPDATA')

    ### PUBLIC METHODS ###

    def get_config_spec(self):
        specs = self.get_option_specs()
        return ['{} = {}'.format(key, value) for key, value in sorted(specs.items())]

    def get_initial_comment(self):
        return [
            '-*- coding: utf-8 -*-',
            ' ',
            'Abjad configuration file, created by Abjad on {}.'.format(time.strftime("%d %B %Y %H:%M:%S")),
            'This file is interpreted by ConfigObj, and should follow ini syntax.',
        ]

    def get_option_comments(self):
        options = self.get_option_definitions()
        comments = [(key, options[key]['comment']) for key in options]
        return dict(comments)

    def get_option_definitions(self):
        options = {
            'abjad_output': {
                'comment': [
                    '',
                    'Set to the one directory where you wish all Abjad generate files',
                    '(such as PDFs, LilyPond, MIDI or log files) to be saved.',
                    'Defaults to $HOME/.abjad/output/'
                ],
                'spec': 'string(default={!r})'.format(
                    os.path.join(self.ABJAD_CONFIG_DIRECTORY_PATH, 'output'))
            },
            'accidental_spelling': { 
                'comment': [
                    '',
                    'Default accidental spelling (mixed|sharps|flats).',
                ],
                'spec': "option('mixed', 'sharps', 'flats', default='mixed')"
            },
            'lilypond_includes': {
                'comment': [
                    '',
                    'Comma-separated list of LilyPond files that Abjad will "\include"',
                    'in all generated *.ly files',
                ],
                'spec': 'list(min=0, default=list())'
            },
            'lilypond_language': {
                'comment': [
                    '',
                    'Language to use in all generated LilyPond files.'
                ],
                'spec': "string(default='english')"
            },
            'lilypond_path': {
                'comment': [
                    '',
                    'Lilypond executable path.  Set to override dynamic lookup.'
                ],
                'spec': "string(default='lilypond')"
            },
            'midi_player': {
                'comment': [
                    '',
                    'MIDI player to play MIDI files with, via play().',
                    'When unset, your environment should know how to open MIDIs.',
                ],
                'spec': "string(default='')"
            },
            'pdf_viewer': {
                'comment': [
                    '',
                    'PDF viewer to view generated PDF files.',
                    'When unset, your environment should know how to open PDFs.',
                ],
                'spec': "string(default='')"
            },
            'text_editor': {
                'comment': [
                    '',
                    'Text editor for viewing text files with (i.e. *.ly).',
                    'When unset, your environment should know how to open text files.'
                ],
                'spec': "string(default='')"
            },

        }
        return options

    def get_option_specs(self):
        options = self.get_option_definitions()
        specs = [(key, options[key]['spec']) for key in options]
        return dict(specs)



from abjad import *
import py.test


def test_Component_start_offset_in_seconds_01():
    '''Offset seconds can not calculate without excplit tempo indication.
    '''

    t = Staff("c'8 d'8 e'8 f'8")

    r'''
    \new Staff {
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert py.test.raises(MissingTempoError, 't[0].start_offset_in_seconds')
    assert py.test.raises(MissingTempoError, 't[0].start_offset_in_seconds')


def test_Component_start_offset_in_seconds_02():
    '''Offset seconds work with explicit tempo indication.
    '''

    t = Staff("c'8 d'8 e'8 f'8")
    contexttools.TempoMark(Duration(1, 8), 48, target_context=Staff)(t)

    r'''
    \new Staff {
        \tempo 8=48
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert t[0].start_offset_in_seconds == Duration(0)
    assert t[1].start_offset_in_seconds == Duration(5, 4)



from abjad.tools import durationtools


def make_multi_measure_rests(durations):
    '''.. versionadded:: 2.0

    Make multi-measure rests from `durations`::

        >>> resttools.make_multi_measure_rests([(4, 4), (7, 4)])
        [MultiMeasureRest('R1'), MultiMeasureRest('R1..')]

    Return list.
    '''
    from abjad.tools import resttools

    multi_measure_rests = []

    for duration in durations:
        written_duration = durationtools.Duration(duration)
        if not written_duration.is_assignable:
            raise AssignabilityError('multi-measure rest durations must be assignable.')
        multi_measure_rest = resttools.MultiMeasureRest(written_duration)
        multi_measure_rests.append(multi_measure_rest)

    return multi_measure_rests



'''Abjad ``measuretools`` package.

    Import regime:
    
    The ``measuretools`` package may import all leaf packages at top level.

    The ``measuretools`` package may import all parent component packages
    at top level. These are ``containertools`` and ``componenttools``.

    The ``measuretools`` package should not import sibling- or cousin-related
    container packages at top level. This includes ``tuplettools`` and other
    packages.

    The ``measuretools`` package must not import ``contexttools`` at top level
    even though measures and time signatures are closely related. This allows
    ``contexttools`` to import the component packages freely at top-level.      

    The ``measuretools`` package must not import ``timesignaturetools`` at top
    level in order to handle ``timesignaturetools`` in parallel to ``contexttools``.

'''
from abjad.tools import importtools

importtools.import_structured_package(__path__[0], globals(), package_root_name='abjad')

_documentation_section = 'core'



from abjad.tools.resttools.Rest import Rest


class MultiMeasureRest(Rest):
    '''.. versionadded:: 2.0

    Abjad model of a multi-measure rest::

        >>> resttools.MultiMeasureRest((1, 4))
        MultiMeasureRest('R4')

    Multi-measure rests are immutable.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    _default_positional_input_arguments = ((1, 4), )

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_representation(self):
        return 'R%s' % self._formatted_duration

    ### PUBLIC PROPERTIES ###

    @property
    def _body(self):
        '''Read-only list of string representation of body of rest.
        Picked up as format contribution at format-time.
        '''
        result = 'R' + str(self._formatted_duration)
        return [result]



import abc
from abjad.tools.datastructuretools.TreeContainer import TreeContainer


class ReSTDirective(TreeContainer):

    ### INITIALIZER ###

    def __init__(self, argument=None, children=None, name=None, options=None):
        TreeContainer.__init__(self, children=children, name=name)
        assert isinstance(options, (dict, type(None)))
        self._argument = argument
        self._options = {}
        if options is not None:
            self._options.update(options)

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _children_rest_format_contributions(self):
        result = []
        for child in self.children:
            result.append('')
            contribution = child._rest_format_contributions
            for x in contribution:
                if x:
                    result.append('   ' + x)
                else:
                    result.append(x)
        return result

    @property
    def _rest_format_contributions(self):
        if self.argument:
            result = ['.. {}:: {}'.format(self.directive, self.argument)]
        else:
            result = ['.. {}::'.format(self.directive)]
        for key, value in sorted(self.options.items()):
            option = '   :{}:'.format(key)
            if value is True:
                pass
            elif value is None or value is False:
                continue
            elif isinstance(value, (list, tuple)):
                option += ' ' + ', '.join(str(x) for x in value)
            elif isinstance(value, (int, float, str)):
                option += ' ' + str(value)
            result.append(option)
        result.extend(self._children_rest_format_contributions)
        return result

    ### READ-ONLY PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def directive(self):
        raise NotImplemented

    @property
    def node_klass(self):
        from abjad.tools import documentationtools
        return (
            documentationtools.ReSTDirective,
            documentationtools.ReSTHeading,
            documentationtools.ReSTHorizontalRule,
            documentationtools.ReSTParagraph,
            )

    @property
    def options(self):
        return self._options

    @property
    def rest_format(self):
        return '\n'.join(self._rest_format_contributions)

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def argument():
        def fget(self):
            return self._argument
        def fset(self, arg):
            assert isinstance(arg, (str, type(None)))
            self._argument = arg
        return property(**locals())



from abjad import *


def test_leaftools_get_nth_leaf_in_thread_from_leaf_01():

    staff = Staff(2 * Voice("c'8 d'8 e'8 f'8"))
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)
    f(staff)

    r'''
    \new Staff {
      \new Voice {
            c'8
            d'8
            e'8
            f'8
      }
      \new Voice {
            g'8
            a'8
            b'8
            c''8
      }
    }
    '''

    leaves = staff.leaves
    assert leaftools.get_nth_leaf_in_thread_from_leaf(staff.leaves[0], 0) is leaves[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(staff.leaves[0], 1) is leaves[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(staff.leaves[0], 2) is leaves[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(staff.leaves[0], 3) is leaves[3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(staff.leaves[0], 4) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(staff.leaves[0], 5) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(staff.leaves[0], 6) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(staff.leaves[0], 7) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(staff.leaves[0], -1) is None


def test_leaftools_get_nth_leaf_in_thread_from_leaf_02():
    '''Voice.
    '''

    t = Voice([Note(i, (1,8)) for i in range(4)])

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[0], 1) is t[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[1], 1) is t[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[2], 1) is t[3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[3], 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[0], -1) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[1], -1) is t[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[2], -1) is t[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[3], -1) is t[2]

    r'''
    \new Voice {
        c'8
        cs'8
        d'8
        ef'8
    }
    '''


def test_leaftools_get_nth_leaf_in_thread_from_leaf_03():
    '''Staff.
    '''

    t = Staff([Note(i, (1,8)) for i in range(4)])

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[0], 1) is t[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[1], 1) is t[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[2], 1) is t[3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[3], 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[0], -1) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[1], -1) is t[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[2], -1) is t[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[3], -1) is t[2]

    r'''
    \new Staff {
        c'8
        cs'8
        d'8
        ef'8
    }
    '''


def test_leaftools_get_nth_leaf_in_thread_from_leaf_04():
    '''Container.
    '''

    t = Container([Note(i, (1,8)) for i in range(4)])

    r'''
    {
        c'8
        cs'8
        d'8
        ef'8
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[0], 1) is t[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[1], 1) is t[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[2], 1) is t[3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[3], 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[0], -1) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[1], -1) is t[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[2], -1) is t[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[3], -1) is t[2]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_05():
    '''Fixed-duration tuplet.
    '''

    t = tuplettools.FixedDurationTuplet(Duration(2,8), [Note(i, (1,8)) for i in range(3)])

    r'''
    \times 2/3 {
        c'8
        cs'8
        d'8
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[0], 1) is t[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[1], 1) is t[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[2], 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[0], -1) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[1], -1) is t[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[2], -1) is t[1]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_06():
    '''Contiguous containers inside a voice.
    '''

    s1 = Container([Note(i, (1,8)) for i in range(4)])
    s2 = Container([Note(i, (1,8)) for i in range(4,8)])
    t = Voice([s1, s2])

    r'''
    \new Voice {
        {
            c'8
            cs'8
            d'8
            ef'8
        }
        {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[0], 1) is s1[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[1], 1) is s1[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[2], 1) is s1[3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[3], 1) is s2[0]

    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[1], -1) is s1[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[2], -1) is s1[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[3], -1) is s1[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0], -1) is s1[3]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_07():
    '''Tuplets inside a voice.
    '''

    t1 = tuplettools.FixedDurationTuplet(Duration(2,8), [Note(i, (1,8)) for i in range(3)])
    t2 = tuplettools.FixedDurationTuplet(Duration(2,8), [Note(i, (1,8)) for i in range(3,6)])
    t = Voice([t1, t2])

    r'''
    \new Voice {
        \times 2/3 {
            c'8
            cs'8
            d'8
        }
        \times 2/3 {
            ef'8
            e'8
            f'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t1[0], 1) is t1[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t1[1], 1) is t1[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t1[2], 1) is t2[0]

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t1[1], -1) is t1[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t1[2], -1) is t1[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t2[0], -1) is t1[2]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_08():
    '''Does not continue across contiguous anonymous voices inside a staff.
    '''

    v1 = Voice([Note(i, (1,8)) for i in range(4)])
    v2 = Voice([Note(i, (1,8)) for i in range(4,8)])
    t = Staff([v1, v2])

    r'''
    \new Staff {
        \new Voice {
            c'8
            cs'8
            d'8
            ef'8
        }
        \new Voice {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[3], 1) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v2[0], -1) is None


def test_leaftools_get_nth_leaf_in_thread_from_leaf_09():
    '''Does cross contiguous equally named voices inside a staff.
    '''

    v1 = Voice([Note(i, (1,8)) for i in range(4)])
    v1.name = 'myvoice'
    v2 = Voice([Note(i, (1,8)) for i in range(4,8)])
    v2.name = 'myvoice'
    t = Staff([v1, v2])

    r'''
    \new Staff {
        \context Voice = "myvoice" {
            c'8
            cs'8
            d'8
            ef'8
        }
        \context Voice = "myvoice" {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[0], 1) is v1[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[1], 1) is v1[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[2], 1) is v1[3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[3], 1) is v2[0]

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[1], -1) is v1[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[2], -1) is v1[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[3], -1) is v1[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v2[0], -1) is v1[3]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_10():
    '''Does not connect through contiguous unequally named voices.
    '''

    v1 = Voice([Note(i, (1,8)) for i in range(4)])
    v1.name = 'yourvoice'
    v2 = Voice([Note(i, (1,8)) for i in range(4,8)])
    v2.name = 'myvoice'
    t = Staff([v1, v2])

    r'''
    \new Staff {
        \context Voice = "yourvoice" {
            c'8
            cs'8
            d'8
            ef'8
        }
        \context Voice = "myvoice" {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[0], 1) is v1[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[1], 1) is v1[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[2], 1) is v1[3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[3], 1) is None

    v2.name = None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[3], 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v2[1], -1) is v2[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v2[2], -1) is v2[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v2[3], -1) is v2[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v2[0], -1) is None


def test_leaftools_get_nth_leaf_in_thread_from_leaf_11():
    '''Does not connect through unequally named staves.
    '''

    v1 = Voice([Note(i, (1,8)) for i in range(4)])
    v1.name = 'low'
    v2 = Voice([Note(i, (1,8)) for i in range(4,8)])
    v2.name = 'low'

    s1 = Staff([v1])
    s1.name = 'mystaff'
    s2 = Staff([v2])
    s2.name = 'mystaff'

    seq = Container([s1, s2])

    r'''
    {
        \context Staff = "mystaff" {
            \context Voice = "low" {
                c'8
                cs'8
                d'8
                ef'8
            }
        }
        \context Staff = "mystaff" {
            \context Voice = "low" {
                e'8
                f'8
                fs'8
                g'8
            }
        }
    }
    '''

    assert not leaftools.get_nth_leaf_in_thread_from_leaf(v1[3], 1) is v2[0]
    assert not leaftools.get_nth_leaf_in_thread_from_leaf(v2[0], -1) is v1[3]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_12():
    '''Does not connect through equally named staves.
    '''

    vl1 = Voice([Note(i, (1,8)) for i in range(4)])
    vl1.name = 'low'
    vl2 = Voice([Note(i, (1,8)) for i in range(4,8)])
    vl2.name = 'low'
    vh1 = Voice([Note(i, (1,8)) for i in range(12,16)])
    vh1.name = 'high'
    vh2 = Voice([Note(i, (1,8)) for i in range(16,20)])
    vh2.name = 'high'

    s1 = Staff([vh1, vl1])
    s1.name = 'mystaff'
    s1.is_parallel = True
    s2 = Staff([vl2, vh2])
    s2.name = 'mystaff'
    s2.is_parallel = True

    seq = Container([s1, s2])

    r'''
    {
        \context Staff = "mystaff" <<
            \context Voice = "high" {
                c''8
                cs''8
                d''8
                ef''8
            }
            \context Voice = "low" {
                c'8
                cs'8
                d'8
                ef'8
            }
        >>
        \context Staff = "mystaff" <<
            \context Voice = "low" {
                e'8
                f'8
                fs'8
                g'8
            }
            \context Voice = "high" {
                e''8
                f''8
                fs''8
                g''8
            }
        >>
    }
    '''

    assert not leaftools.get_nth_leaf_in_thread_from_leaf(vl1[3], 1) is vl2[0]
    assert not leaftools.get_nth_leaf_in_thread_from_leaf(vh1[3], 1) is vh2[0]

    assert not leaftools.get_nth_leaf_in_thread_from_leaf(vl2[0], 1) is vl1[0]
    assert not leaftools.get_nth_leaf_in_thread_from_leaf(vh2[0], 1) is vh1[3]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_13():
    '''Does connect through symmetrical nested containers in a voice.
    '''

    s1 = Container([Note(i, (1,8)) for i in range(4)])
    s1 = Container([s1])
    s2 = Container([Note(i, (1,8)) for i in range(4,8)])
    s2 = Container([s2])
    t = Voice([s1, s2])

    r'''
    \new Voice {
        {
            {
                c'8
                cs'8
                d'8
                ef'8
            }
        }
        {
            {
                e'8
                f'8
                fs'8
                g'8
            }
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[0][0], 1) is s1[0][1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[0][1], 1) is s1[0][2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[0][2], 1) is s1[0][3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[0][3], 1) is s2[0][0]

    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0][1], -1) is s2[0][0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0][2], -1) is s2[0][1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0][3], -1) is s2[0][2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0][0], -1) is s1[0][3]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_14():
    '''Tautological parentage asymmetries result in symmetric (balanced) threaded parentage.
    '''

    s1 = Container([Note(i, (1,8)) for i in range(4)])
    s2 = Container([Note(i, (1,8)) for i in range(4,8)])
    s2 = Container([s2])
    s2 = Container([s2])
    t = Voice([s1, s2])

    r'''
    \new Voice {
        {
            {
                c'8
                cs'8
                d'8
                ef'8
            }
        }
        {
            {
                e'8
                f'8
                fs'8
                g'8
            }
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[0], 1) is s1[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[1], 1) is s1[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[2], 1) is s1[3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[3], 1) is s2[0][0][0]

    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0][0][1], -1) is s2[0][0][0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0][0][2], -1) is s2[0][0][1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0][0][3], -1) is s2[0][0][2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0][0][0], -1) is s1[3]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_15():
    '''Tautological parentage asymmetries result in symmetric (balanced) threaded parentage.
    '''

    s1 = Container([Note(i, (1,8)) for i in range(4)])
    s1 = Container([s1])
    s1 = Container([s1])
    s2 = Container([Note(i, (1,8)) for i in range(4,8)])
    t = Voice([s1, s2])

    r'''
    \new Voice {
        {
            {
                {
                    c'8
                    cs'8
                    d'8
                    ef'8
                }
            }
        }
        {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[0][0][0], 1) is s1[0][0][1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[0][0][1], 1) is s1[0][0][2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[0][0][2], 1) is s1[0][0][3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[0][0][3], 1) is s2[0]

    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0], -1) is s1[0][0][3]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[1], -1) is s2[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[2], -1) is s2[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[3], -1) is s2[2]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_16():
    '''Does connect in sequence of alternating containers and notes.
    '''

    s1 = Container([Note(i, (1,8)) for i in range(2)])
    s2 = Container([Note(i, (1,8)) for i in range(3,5)])
    v = Voice([s1, Note(2, (1,8)), s2])

    r'''
    \new Voice {
        {
            c'8
            cs'8
        }
        d'8
        {
            ef'8
            e'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(s1[1], 1) is v[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v[1], 1) is s2[0]

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v[1], -1) is s1[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(s2[0], -1) is v[1]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_17():
    '''Does connect in sequence of alternating tuplets and notes.
    '''

    t1 = tuplettools.FixedDurationTuplet(Duration(1,4), [Note(i, (1,8)) for i in range(3)])
    t2 = tuplettools.FixedDurationTuplet(Duration(1,4), [Note(i, (1,8)) for i in range(4,7)])
    v = Voice([t1, Note(3, (1,8)), t2])

    r'''
    \new Voice {
        \times 2/3 {
            c'8
            cs'8
            d'8
        }
        ef'8
        \times 2/3 {
            e'8
            f'8
            fs'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t1[-1], 1) is v[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v[1], 1) is t2[0]

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v[1], -1) is t1[-1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t2[0], -1) is v[1]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_18():
    '''Does connect through asymmetrically nested tuplets.
    '''

    tinner = tuplettools.FixedDurationTuplet(Duration(1, 4), Note(0, (1, 8)) * 3)
    t = tuplettools.FixedDurationTuplet(Duration(2, 4), [Note("c'4"), tinner, Note("c'4")])

    r'''
    \times 2/3 {
        c'4
        \times 2/3 {
            c'8
            c'8
            c'8
        }
        c'4
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[0], 1) is tinner[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(tinner[-1], 1) is t[-1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(t[-1], -1) is tinner[-1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(tinner[0], -1) is t[0]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_19():
    '''Return none in asymmetric thread parentage structures.
    '''

    v1 = Voice([Note(i , (1,8)) for i in range(3)])
    n = Note(3, (1,8))
    v2 = Voice([Note(i , (1,8)) for i in range(4,8)])
    t = Staff([v1, n, v2])

    r'''
    \new Staff {
        \new Voice {
            c'8
            cs'8
            d'8
        }
        ef'8
        \new Voice {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[-1], 1) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(n, 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v2[0], -1) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(n, -1) is None


def test_leaftools_get_nth_leaf_in_thread_from_leaf_20():
    '''Non-contiguous or broken threads do not connect.
    '''

    v1 = Voice([Note(i , (1,8)) for i in range(3)])
    v1.name = 'myvoice'
    v2 = Voice([Note(i , (1,8)) for i in range(4,8)])
    v2.name = 'yourvoice'
    v3 = Voice([Note(i , (1,8)) for i in range(4,8)])
    v3.name = 'myvoice'
    t = Staff([v1, v2, v3])

    r'''
    \new Staff {
        \context Voice = "myvoice" {
            c'8
            cs'8
            d'8
        }
        \context Voice = "yourvoice" {
            e'8
            f'8
            fs'8
            g'8
        }
        \context Voice = "myvoice" {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[-1], 1) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v2[-1], 1) is None

    v2.name = None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v1[-1], 1) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v2[-1], 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(v3[0], -1) is None
    assert leaftools.get_nth_leaf_in_thread_from_leaf(v2[0], -1) is None


def test_leaftools_get_nth_leaf_in_thread_from_leaf_21():
    '''Does not connect through nested anonymous voices.
    '''

    vin = Voice([Note(i, (1,8)) for i in range(3)])
    vout = Voice([vin, Note(3, (1,8))])

    r'''
    \new Voice {
        \new Voice {
            c'8
            cs'8
            d'8
        }
        ef'8
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[0], 1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], 1) is vin[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[2], 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], -1) is vin[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[2], -1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vout[1], -1) is None


def test_leaftools_get_nth_leaf_in_thread_from_leaf_22():
    '''Does not connect through nested anonymous voices.
    '''

    vin = Voice([Note(i, (1,8)) for i in range(1,4)])
    vout = Voice([Note(0, (1,8)), vin])

    r'''
    \new Voice {
        c'8
        \new Voice {
            cs'8
            d'8
            ef'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[0], 1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], 1) is vin[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vout[0], 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], -1) is vin[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[2], -1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[0], -1) is None



def test_leaftools_get_nth_leaf_in_thread_from_leaf_23():
    '''Does connect through nested equally named voices.
    '''

    vin = Voice([Note(i, (1,8)) for i in range(3)])
    vin.name = 'myvoice'
    vout = Voice([vin, Note(3, (1,8))])
    vout.name = 'myvoice'

    r'''
    \context Voice = "myvoice" {
        \context Voice = "myvoice" {
            c'8
            cs'8
            d'8
        }
        ef'8
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[0], 1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], 1) is vin[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[2], 1) is vout[1]

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], -1) is vin[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[2], -1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vout[1], -1) is vin[-1]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_24():
    '''Does connect through nested equally named voices.
    '''

    vin = Voice([Note(i, (1,8)) for i in range(1,4)])
    vin.name = 'myvoice'
    vout = Voice([Note(0, (1,8)), vin])
    vout.name = 'myvoice'

    r'''
    \context Voice = "myvoice" {
        c'8
        \context Voice = "myvoice" {
            cs'8
            d'8
            ef'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[0], 1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], 1) is vin[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vout[0], 1) is vin[0]

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], -1) is vin[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[2], -1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[0], -1) is vout[0]


def test_leaftools_get_nth_leaf_in_thread_from_leaf_25():
    '''Return none on nested *differently* named voices.
    '''

    vin = Voice([Note(i, (1,8)) for i in range(3)])
    vin.name = 'yourvoice'
    vout = Voice([vin, Note(3, (1,8))])
    vout.name = 'myvoice'

    r'''
    \context Voice = "myvoice" {
        \context Voice = "yourvoice" {
            c'8
            cs'8
            d'8
        }
        ef'8
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[0], 1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], 1) is vin[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[2], 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], -1) is vin[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[2], -1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vout[1], -1) is None


def test_leaftools_get_nth_leaf_in_thread_from_leaf_26():
    '''Return none on nested *differently* named Voices.
    '''

    vin = Voice([Note(i, (1,8)) for i in range(1, 4)])
    vin.name = 'yourvoice'
    vout = Voice([Note(0, (1,8)), vin])
    vout.name = 'myvoice'

    r'''
    \context Voice = "myvoice" {
        c'8
        \context Voice = "yourvoice" {
            cs'8
            d'8
            ef'8
        }
    }
    '''

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[0], 1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], 1) is vin[2]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vout[0], 1) is None

    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[1], -1) is vin[0]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vin[2], -1) is vin[1]
    assert leaftools.get_nth_leaf_in_thread_from_leaf(vout[1], -1) is None
